import subprocess
from unittest.mock import ANY

import pytest

from briefcase.integrations.subprocess import CommandOutputParseError, ParseError


def splitlines_parser(data):
    """A test parser that returns the input data, split by line."""
    return data.splitlines()


def second_line_parser(data):
    """A test parser that returns the second line of input."""
    try:
        return data.splitlines()[1]
    except IndexError:
        raise ParseError("Input does not contain 2 lines")


def third_line_parser(data):
    """A test parser that returns the third line of input."""
    try:
        return data.splitlines()[2]
    except IndexError:
        raise ParseError("Input does not contain 3 lines")


def test_call(mock_sub, capsys, sub_check_output_kw):
    """A simple call to check_output will be invoked."""

    output = mock_sub.parse_output(splitlines_parser, ["hello", "world"])

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        **sub_check_output_kw,
    )
    assert capsys.readouterr().out == ""
    assert output == ["some output line 1", "more output line 2"]


def test_call_with_arg(mock_sub, capsys, sub_check_output_kw):
    """Any extra keyword arguments are passed through as-is to check_output."""

    output = mock_sub.parse_output(
        splitlines_parser,
        ["hello", "world"],
        extra_kw="extra",
    )

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        extra_kw="extra",
        **sub_check_output_kw,
    )
    assert capsys.readouterr().out == ""
    assert output == ["some output line 1", "more output line 2"]


def test_call_with_parser_success(mock_sub, capsys, sub_check_output_kw):
    """Parser returns expected portion of check_output's output."""

    output = mock_sub.parse_output(second_line_parser, ["hello", "world"])

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        **sub_check_output_kw,
    )

    assert output == "more output line 2"


def test_call_with_parser_error(mock_sub, capsys, sub_check_output_kw):
    """Parser errors on output from check_output."""

    with pytest.raises(
        CommandOutputParseError,
        match="Unable to parse command output: Input does not contain 3 lines",
    ):
        mock_sub.parse_output(third_line_parser, ["hello", "world"])

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        **sub_check_output_kw,
    )

    expected_output = (
        "\n"
        "Command Output Parsing Error:\n"
        "    Input does not contain 3 lines\n"
        "Command:\n"
        "    hello world\n"
        "Command Output:\n"
        "    some output line 1\n"
        "    more output line 2\n"
    )
    assert capsys.readouterr().out == expected_output


@pytest.mark.parametrize(
    "in_kwargs, kwargs",
    [
        ({}, {"text": True, "encoding": ANY, "errors": "backslashreplace"}),
        ({"text": True}, {"text": True, "encoding": ANY, "errors": "backslashreplace"}),
        ({"text": False}, {"text": False}),
        ({"universal_newlines": False}, {"text": False}),
        (
            {"universal_newlines": True},
            {"text": True, "encoding": ANY, "errors": "backslashreplace"},
        ),
    ],
)
def test_text_eq_true_default_overriding(mock_sub, in_kwargs, kwargs):
    """If text or universal_newlines is explicitly provided, those should override
    text=true default and universal_newlines should be converted to text."""

    mock_sub.parse_output(splitlines_parser, ["hello", "world"], **in_kwargs)

    mock_sub._subprocess.check_output.assert_called_with(
        ["hello", "world"],
        stderr=subprocess.STDOUT,
        **kwargs,
    )



import pytest

from briefcase.platforms.android.gradle import safe_formal_name


@pytest.mark.parametrize(
    "formal_name, safe_name",
    [
        ("Hello World", "Hello World"),
        # The invalid list is all stripped
        ("Hello/World/", "HelloWorld"),
        ("Hello\\World", "HelloWorld"),
        ("Hello:World", "HelloWorld"),
        ("Hello<World", "HelloWorld"),
        ("Hello>World", "HelloWorld"),
        ('Hello "World"', "Hello World"),
        ("Hello World?", "Hello World"),
        ("Hello|World", "HelloWorld"),
        ("Hello World!", "Hello World"),
        # All invalid punctuation is removed
        # Valid punctuation is preserved
        ("Hello! (World?)", "Hello (World)"),
        # Position of punctuation doesn't matter
        ("Hello! World", "Hello World"),
        ("!Hello World", "Hello World"),
        # If removing punctuation leads to double spaces, reduce the double spaces
        ("Hello | World", "Hello World"),
        ("Hello World |", "Hello World"),
        ("| Hello World", "Hello World"),
    ],
)
def test_safe_formal_name(formal_name, safe_name):
    assert safe_formal_name(formal_name) == safe_name



import os
import subprocess
from pathlib import Path

import pytest

from briefcase.exceptions import BriefcaseCommandError

from ...utils import create_file, create_installed_package, file_content


@pytest.mark.parametrize("pre_existing", [True, False])
def test_merge(dummy_command, pre_existing, tmp_path):
    "Multiple source folders can be merged"
    if pre_existing:
        # Create some pre-existing package content. This should all be deleted as a
        # result of the merge process.
        create_installed_package(tmp_path / "merged_app_packages", "legacy")

    # Create 2 packages in the "gothic" architecture app package sources
    create_installed_package(tmp_path / "app_packages.gothic", "first", "1.2.3")
    create_installed_package(
        tmp_path / "app_packages.gothic",
        "second",
        "2.3.4",
        tag="macOS_11_0_gothic",
        extra_content=[
            ("second/other.py", "# other python"),
            ("second/different.py", "# different python"),
            ("second/some-binary", "# A file with executable permissions", 0o755),
            ("second/sub1/module1.dylib", "dylib-gothic"),
            ("second/sub1/module2.so", "dylib-gothic"),
            ("second/sub1/module3.dylib", "dylib-gothic"),
        ],
    )

    # Create 2 packages in the "modern" architecture app package sources
    # The first package is pure, so it won't exist in the second app_packages.
    # The "second" package:
    # - is missing the "other" python file and "module3" dylib
    # - has a "module4" dylib and an "extra" python file in a unique folder.
    create_installed_package(
        tmp_path / "app_packages.modern",
        "second",
        "2.3.4",
        tag="macOS_11_0_modern",
        extra_content=[
            ("second/different.py", "# I need to be different"),
            ("second/sub1/module1.dylib", "dylib-modern"),
            ("second/sub1/module2.so", "dylib-modern"),
            ("second/sub1/module4.dylib", "dylib-modern"),
            ("second/sub2/extra.py", "# extra python"),
        ],
    )

    # Mock subprocess so that lipo generates output files.
    def lipo(cmd, **kwargs):
        if cmd[0] != "lipo":
            pytest.fail(f"Subprocess called {cmd[0]}, not lipo")

        create_file(cmd[3], "dylib-merged")

    dummy_command.tools.subprocess.run.side_effect = lipo

    # Merge the two sources into a final location.
    merged_path = tmp_path / "merged_app_packages"
    dummy_command.merge_app_packages(
        merged_path,
        sources=[
            tmp_path / "app_packages.gothic",
            tmp_path / "app_packages.modern",
        ],
    )

    # The final merged app packages contains only the merged content.

    assert {
        (path.relative_to(merged_path), file_content(path))
        for path in merged_path.glob("**/*")
    } == {
        (Path("first"), None),
        (Path("first/__init__.py"), ""),
        (Path("first/app.py"), "# This is the app"),
        (Path("first-1.2.3.dist-info"), None),
        (Path("first-1.2.3.dist-info/INSTALLER"), "pip\n"),
        (
            Path("first-1.2.3.dist-info/METADATA"),
            "\n".join(
                [
                    "Metadata-Version: 2.1",
                    "Name: first",
                    "Version: 1.2.3",
                    "Summary: A packaged named first.",
                    "Author-email: Jane Developer <jane@example.com>",
                    "\n",
                ]
            ),
        ),
        (Path("first-1.2.3.dist-info/RECORD"), ""),
        (
            Path("first-1.2.3.dist-info/WHEEL"),
            "\n".join(
                [
                    "Wheel-Version: 1.0",
                    "Generator: test-case",
                    "Root-Is-Purelib: true",
                    "Tag: py3-none-any",
                    "\n",
                ]
            ),
        ),
        (Path("second"), None),
        (Path("second/__init__.py"), ""),
        (Path("second/app.py"), "# This is the app"),
        (Path("second/different.py"), "# different python"),
        (Path("second/some-binary"), "# A file with executable permissions"),
        (Path("second/other.py"), "# other python"),
        (Path("second/sub1"), None),
        (Path("second/sub1/module1.dylib"), "dylib-merged"),
        (Path("second/sub1/module2.so"), "dylib-merged"),
        (Path("second/sub1/module3.dylib"), "dylib-merged"),
        (Path("second/sub1/module4.dylib"), "dylib-merged"),
        (Path("second/sub2"), None),
        (Path("second/sub2/extra.py"), "# extra python"),
        (Path("second-2.3.4.dist-info"), None),
        (Path("second-2.3.4.dist-info/INSTALLER"), "pip\n"),
        (
            Path("second-2.3.4.dist-info/METADATA"),
            "\n".join(
                [
                    "Metadata-Version: 2.1",
                    "Name: second",
                    "Version: 2.3.4",
                    "Summary: A packaged named second.",
                    "Author-email: Jane Developer <jane@example.com>",
                    "\n",
                ]
            ),
        ),
        (Path("second-2.3.4.dist-info/RECORD"), ""),
        (
            Path("second-2.3.4.dist-info/WHEEL"),
            "\n".join(
                [
                    "Wheel-Version: 1.0",
                    "Generator: test-case",
                    "Root-Is-Purelib: false",
                    # The first source wins
                    "Tag: macOS_11_0_gothic",
                    "\n",
                ]
            ),
        ),
    }

    # Check that the embedded binary has executable permissions
    assert os.access(merged_path / "second/some-binary", os.X_OK)


def test_merge_problem(dummy_command, tmp_path):
    "If a binary cannot be merged, an exception is raised."

    # Create 2 packages in the "gothic" architecture app package sources
    create_installed_package(tmp_path / "app_packages.gothic", "first", "1.2.3")
    create_installed_package(
        tmp_path / "app_packages.gothic",
        "second",
        "2.3.4",
        tag="macOS_11_0_gothic",
        extra_content=[
            ("second/sub1/module1.dylib", "dylib-gothic"),
        ],
    )
    # Create 2 packages in the "modern" architecture app package sources
    # The first package is pure, so it won't exist in the second app_packages.
    # The "second" package:
    # - is missing the "other" python file and "module3" dylib
    # - has a "module4" dylib and an "extra" python file in a unique folder.
    create_installed_package(
        tmp_path / "app_packages.modern",
        "second",
        "2.3.4",
        tag="macOS_11_0_modern",
        extra_content=[
            ("second/sub1/module1.dylib", "dylib-modern"),
        ],
    )

    # Mock subprocess so that lipo generates an exception
    dummy_command.tools.subprocess.run.side_effect = subprocess.CalledProcessError(
        returncode=1, cmd="lipo"
    )

    # Merge the two sources into a final location. This will raise an exception.
    with pytest.raises(BriefcaseCommandError, match=r""):
        merged_path = tmp_path / "merged_app_packages"
        dummy_command.merge_app_packages(
            merged_path,
            sources=[
                tmp_path / "app_packages.gothic",
                tmp_path / "app_packages.modern",
            ],
        )


def test_merge_no_dylib(dummy_command, tmp_path, capsys):
    "If there are no dylibs, no merging is performed"

    # Create 2 pure python packages in the first app_packages folder.
    create_installed_package(tmp_path / "app_packages.gothic", "first", "1.2.3")
    create_installed_package(tmp_path / "app_packages.gothic", "second", "2.3.4")
    # Create an empty second app_packages folder.
    (tmp_path / "app_packages.modern").mkdir()

    # Merge the two sources into a final location.
    merged_path = tmp_path / "merged_app_packages"
    dummy_command.merge_app_packages(
        merged_path,
        sources=[
            tmp_path / "app_packages.gothic",
            tmp_path / "app_packages.modern",
        ],
    )

    # subprocess wasn't called.
    dummy_command.tools.subprocess.run.assert_not_called()

    # The final merged app packages contains only the merged content.
    assert {
        (path.relative_to(merged_path), file_content(path))
        for path in merged_path.glob("**/*")
    } == {
        (Path("first"), None),
        (Path("first/__init__.py"), ""),
        (Path("first/app.py"), "# This is the app"),
        (Path("first-1.2.3.dist-info"), None),
        (Path("first-1.2.3.dist-info/INSTALLER"), "pip\n"),
        (
            Path("first-1.2.3.dist-info/METADATA"),
            "\n".join(
                [
                    "Metadata-Version: 2.1",
                    "Name: first",
                    "Version: 1.2.3",
                    "Summary: A packaged named first.",
                    "Author-email: Jane Developer <jane@example.com>",
                    "\n",
                ]
            ),
        ),
        (Path("first-1.2.3.dist-info/RECORD"), ""),
        (
            Path("first-1.2.3.dist-info/WHEEL"),
            "\n".join(
                [
                    "Wheel-Version: 1.0",
                    "Generator: test-case",
                    "Root-Is-Purelib: true",
                    "Tag: py3-none-any",
                    "\n",
                ]
            ),
        ),
        (Path("second"), None),
        (Path("second/__init__.py"), ""),
        (Path("second/app.py"), "# This is the app"),
        (Path("second-2.3.4.dist-info"), None),
        (Path("second-2.3.4.dist-info/INSTALLER"), "pip\n"),
        (
            Path("second-2.3.4.dist-info/METADATA"),
            "\n".join(
                [
                    "Metadata-Version: 2.1",
                    "Name: second",
                    "Version: 2.3.4",
                    "Summary: A packaged named second.",
                    "Author-email: Jane Developer <jane@example.com>",
                    "\n",
                ]
            ),
        ),
        (Path("second-2.3.4.dist-info/RECORD"), ""),
        (
            Path("second-2.3.4.dist-info/WHEEL"),
            "\n".join(
                [
                    "Wheel-Version: 1.0",
                    "Generator: test-case",
                    "Root-Is-Purelib: true",
                    "Tag: py3-none-any",
                    "\n",
                ]
            ),
        ),
    }



from unittest.mock import MagicMock

import pytest

from briefcase.integrations.cookiecutter import RGBExtension


@pytest.mark.parametrize(
    "value, red, green, blue",
    [
        ("#000000", 0.0, 0.0, 0.0),
        ("#FFFFFF", 1.0, 1.0, 1.0),
        ("#336699", 0.2, 0.4, 0.6),
        ("#abcdef", 0.6705882, 0.8039215, 0.9372549),
        # Not a color RGB hexstring
        ("hoovaloo", 1.0, 1.0, 1.0),
    ],
)
def test_py_tag(value, red, green, blue):
    env = MagicMock()
    env.filters = {}
    RGBExtension(env)
    assert env.filters["float_red"](value) == pytest.approx(red)
    assert env.filters["float_green"](value) == pytest.approx(green)
    assert env.filters["float_blue"](value) == pytest.approx(blue)



# Xcode uses the same run implementation as the base app;
# Run a basic test to ensure coverage, but fall back to
# the app backend for exhaustive tests.
import subprocess
from signal import SIGTERM
from unittest import mock

import pytest

from briefcase.console import Console, Log
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.macOS import macOS_log_clean_filter
from briefcase.platforms.macOS.xcode import macOSXcodeRunCommand


@pytest.fixture
def run_command(tmp_path):
    command = macOSXcodeRunCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.home_path = tmp_path / "home"
    command.tools.subprocess = mock.MagicMock(spec_set=Subprocess)
    command._stream_app_logs = mock.MagicMock()

    # To satisfy coverage, the stop function must be invoked
    # at least once when streaming app logs.
    def mock_stream_app_logs(app, stop_func, **kwargs):
        stop_func()

    command._stream_app_logs.side_effect = mock_stream_app_logs
    command.tools.os.kill = mock.MagicMock()

    return command


def test_run_app(run_command, first_app_config, sleep_zero, tmp_path, monkeypatch):
    """A macOS Xcode app can be started."""
    # Mock a popen object that represents the log stream
    log_stream_process = mock.MagicMock(spec_set=subprocess.Popen)
    run_command.tools.subprocess.Popen.return_value = log_stream_process

    # Monkeypatch the tools get the process ID
    monkeypatch.setattr(
        "briefcase.platforms.macOS.get_process_id_by_command", lambda *a, **kw: 100
    )

    run_command.run_app(first_app_config, test_mode=False, passthrough=[])

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path],
        cwd=tmp_path / "home",
        check=True,
    )

    # The log stream was started
    run_command._stream_app_logs.assert_called_with(
        first_app_config,
        popen=log_stream_process,
        test_mode=False,
        clean_filter=macOS_log_clean_filter,
        clean_output=True,
        stop_func=mock.ANY,
        log_stream=True,
    )

    # The app process was killed on exit.
    run_command.tools.os.kill.assert_called_with(100, SIGTERM)


def test_run_app_with_passthrough(
    run_command,
    first_app_config,
    sleep_zero,
    tmp_path,
    monkeypatch,
):
    """A macOS Xcode app can be started with args."""
    # Mock a popen object that represents the log stream
    log_stream_process = mock.MagicMock(spec_set=subprocess.Popen)
    run_command.tools.subprocess.Popen.return_value = log_stream_process

    # Monkeypatch the tools get the process ID
    monkeypatch.setattr(
        "briefcase.platforms.macOS.get_process_id_by_command", lambda *a, **kw: 100
    )

    # Run the app with args
    run_command.run_app(
        first_app_config,
        test_mode=False,
        passthrough=["foo", "--bar"],
    )

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path, "--args", "foo", "--bar"],
        cwd=tmp_path / "home",
        check=True,
    )

    # The log stream was started
    run_command._stream_app_logs.assert_called_with(
        first_app_config,
        popen=log_stream_process,
        test_mode=False,
        clean_filter=macOS_log_clean_filter,
        clean_output=True,
        stop_func=mock.ANY,
        log_stream=True,
    )

    # The app process was killed on exit.
    run_command.tools.os.kill.assert_called_with(100, SIGTERM)


def test_run_app_test_mode(
    run_command,
    first_app_config,
    sleep_zero,
    tmp_path,
    monkeypatch,
):
    """A macOS Xcode app can be started in test mode."""
    # Mock a popen object that represents the log stream
    log_stream_process = mock.MagicMock(spec_set=subprocess.Popen)
    run_command.tools.subprocess.Popen.return_value = log_stream_process

    # Monkeypatch the tools get the process ID
    monkeypatch.setattr(
        "briefcase.platforms.macOS.get_process_id_by_command", lambda *a, **kw: 100
    )

    run_command.run_app(first_app_config, test_mode=True, passthrough=[])

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path],
        cwd=tmp_path / "home",
        check=True,
        env={"BRIEFCASE_MAIN_MODULE": "tests.first_app"},
    )

    # The log stream was started
    run_command._stream_app_logs.assert_called_with(
        first_app_config,
        popen=log_stream_process,
        test_mode=True,
        clean_filter=macOS_log_clean_filter,
        clean_output=True,
        stop_func=mock.ANY,
        log_stream=True,
    )

    # The app process was killed on exit.
    run_command.tools.os.kill.assert_called_with(100, SIGTERM)


def test_run_app_test_mode_with_passthrough(
    run_command,
    first_app_config,
    sleep_zero,
    tmp_path,
    monkeypatch,
):
    """A macOS Xcode app can be started in test mode with args."""
    # Mock a popen object that represents the log stream
    log_stream_process = mock.MagicMock(spec_set=subprocess.Popen)
    run_command.tools.subprocess.Popen.return_value = log_stream_process

    # Monkeypatch the tools get the process ID
    monkeypatch.setattr(
        "briefcase.platforms.macOS.get_process_id_by_command", lambda *a, **kw: 100
    )

    # Run app in test mode with args
    run_command.run_app(
        first_app_config,
        test_mode=True,
        passthrough=["foo", "--bar"],
    )

    # Calls were made to start the app and to start a log stream.
    bin_path = run_command.binary_path(first_app_config)
    sender = bin_path / "Contents/MacOS/First App"
    run_command.tools.subprocess.Popen.assert_called_with(
        [
            "log",
            "stream",
            "--style",
            "compact",
            "--predicate",
            f'senderImagePath=="{sender}"'
            f' OR (processImagePath=="{sender}"'
            ' AND senderImagePath=="/usr/lib/libffi.dylib")',
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        bufsize=1,
    )
    run_command.tools.subprocess.run.assert_called_with(
        ["open", "-n", bin_path, "--args", "foo", "--bar"],
        cwd=tmp_path / "home",
        check=True,
        env={"BRIEFCASE_MAIN_MODULE": "tests.first_app"},
    )

    # The log stream was started
    run_command._stream_app_logs.assert_called_with(
        first_app_config,
        popen=log_stream_process,
        test_mode=True,
        clean_filter=macOS_log_clean_filter,
        clean_output=True,
        stop_func=mock.ANY,
        log_stream=True,
    )

    # The app process was killed on exit.
    run_command.tools.os.kill.assert_called_with(100, SIGTERM)



import os
import subprocess
import sys
from unittest.mock import MagicMock

import pytest

from briefcase.console import Console, Log
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.iOS.xcode import iOSXcodeOpenCommand

from ....utils import create_file


@pytest.fixture
def open_command(tmp_path, first_app_config):
    command = iOSXcodeOpenCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.os = MagicMock(spec_set=os)
    command.tools.subprocess = MagicMock(spec_set=Subprocess)

    # Mock the call to verify the existence of the cmdline tools
    command.tools.subprocess.check_output.side_effect = [
        # xcode-select -p
        "/Applications/Xcode.app/Contents/Developer",
        # xcodebuild -version
        "Xcode 13.0.0",
        # xcode-select  --install
        subprocess.CalledProcessError(cmd=["xcode-select", "--install"], returncode=1),
        # /usr/bin/clang --version
        "Apple clang version 13.1.6 (clang-1316.0.21.2.5)\n...",
    ]
    return command


@pytest.mark.skipif(sys.platform != "darwin", reason="macOS specific test")
def test_open(open_command, first_app_config, tmp_path):
    """On iOS, Open starts Xcode on the project."""
    # Create the project file to mock a created project.
    create_file(
        open_command.project_path(first_app_config) / "project.pbxproj",
        "Xcode project",
    )

    open_command(first_app_config)

    open_command.tools.subprocess.Popen.assert_called_once_with(
        [
            "open",
            tmp_path
            / "base_path"
            / "build"
            / "first-app"
            / "ios"
            / "xcode"
            / "First App.xcodeproj",
        ]
    )



import shutil
from unittest import mock


def test_no_source(create_command, tmp_path):
    """If the app doesn't define a source, no image is installed."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Try to install the image from no source.
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source=None,
        variant=None,
        size=None,
        target=out_path,
    )

    # No file was installed.
    create_command.tools.shutil.copy.assert_not_called()


def test_no_source_with_size(create_command, tmp_path):
    """If the app doesn't define a source, and a size is requested, no image is
    installed."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Try to install the image from no source.
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source=None,
        variant=None,
        size="3742",
        target=out_path,
    )

    # No file was installed.
    create_command.tools.shutil.copy.assert_not_called()


def test_no_requested_size(create_command, tmp_path, capsys):
    """If the app specifies a no-size image, an un-annotated image is used."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source="input/original",
        variant=None,
        size=None,
        target=out_path,
    )

    # The right message was written to output
    expected = (
        "Installing input/original.png as sample image... started\n"
        "Installing input/original.png as sample image... done\n\n"
    )
    assert capsys.readouterr().out == expected

    # The file was copied into position
    create_command.tools.shutil.copy.assert_called_with(
        create_command.base_path / "input/original.png",
        out_path,
    )


def test_no_requested_size_invalid_path(create_command, tmp_path, capsys):
    """If the app specifies a no-size image that doesn't exist, an error is raised."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.copy.side_effect = FileNotFoundError

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source="input/original",
        variant=None,
        size=None,
        target=out_path,
    )

    # The right message was written to output
    expected = "Unable to find input/original.png for sample image; using default\n"
    assert capsys.readouterr().out == expected

    # The file was not copied
    assert create_command.tools.shutil.copy.call_count == 0


def test_requested_size(create_command, tmp_path, capsys):
    """If the app specifies a sized image, an annotated image filename is used."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original-3742.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source="input/original",
        variant=None,
        size="3742",
        target=out_path,
    )

    # The right message was written to output
    expected = (
        "Installing input/original-3742.png as 3742px sample image... started\n"
        "Installing input/original-3742.png as 3742px sample image... done\n\n"
    )
    assert capsys.readouterr().out == expected

    # The file was copied into position
    create_command.tools.shutil.copy.assert_called_with(
        create_command.base_path / "input/original-3742.png",
        out_path,
    )


def test_requested_size_invalid_path(create_command, tmp_path, capsys):
    """If the app specifies a sized image that doesn't exist, an error is raised."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.copy.side_effect = FileNotFoundError

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source="input/original",
        variant=None,
        size="3742",
        target=out_path,
    )

    # The right message was written to output
    expected = "Unable to find input/original-3742.png for 3742px sample image; using default\n"
    assert capsys.readouterr().out == expected

    # The file was not copied
    assert create_command.tools.shutil.copy.call_count == 0


def test_variant_with_no_requested_size(create_command, tmp_path, capsys):
    """If the app specifies a variant with no size, the variant is used unsized."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source={
            "round": "input/original",
        },
        variant="round",
        size=None,
        target=out_path,
    )

    # The right message was written to output
    expected = (
        "Installing input/original.png as round sample image... started\n"
        "Installing input/original.png as round sample image... done\n\n"
    )
    assert capsys.readouterr().out == expected

    # The file was copied into position
    create_command.tools.shutil.copy.assert_called_with(
        create_command.base_path / "input/original.png",
        out_path,
    )


def test_variant_without_variant_source_and_no_requested_size(
    create_command, tmp_path, capsys
):
    """If the template specifies a variant with no size, but app doesn't define
    variants, the default construction is used."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source="input/original",
        variant="round",
        size=None,
        target=out_path,
    )

    # The right message was written to output
    expected = "Unable to find input/original-round.png for round sample image; using default\n"
    assert capsys.readouterr().out == expected

    # No file was installed.
    create_command.tools.shutil.copy.assert_not_called()


def test_unknown_variant_with_no_requested_size(create_command, tmp_path, capsys):
    """If the app specifies an unknown variant, a message is reported."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source={
            "round": "input/original",
        },
        variant="unknown",
        size=None,
        target=out_path,
    )

    # The right message was written to output
    expected = "Unknown variant 'unknown' for sample image; using default\n"
    assert capsys.readouterr().out == expected

    # No file was installed.
    create_command.tools.shutil.copy.assert_not_called()


def test_variant_with_size(create_command, tmp_path, capsys):
    """If the app specifies a variant with a size, the sized variant is used."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original-3742.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source={
            "round": "input/original",
        },
        variant="round",
        size="3742",
        target=out_path,
    )

    # The right message was written to output
    expected = (
        "Installing input/original-3742.png as 3742px round sample image... started\n"
        "Installing input/original-3742.png as 3742px round sample image... done\n\n"
    )
    assert capsys.readouterr().out == expected

    # The file was copied into position
    create_command.tools.shutil.copy.assert_called_with(
        create_command.base_path / "input/original-3742.png",
        out_path,
    )


def test_variant_with_size_without_variants(create_command, tmp_path, capsys):
    """If the app specifies a variant with a size, but no variants are specified, a
    message is output."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original-3742.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source="input/original",
        variant="round",
        size="3742",
        target=out_path,
    )

    # The right message was written to output
    expected = "Unable to find input/original-round-3742.png for 3742px round sample image; using default\n"
    assert capsys.readouterr().out == expected

    # No file was installed.
    create_command.tools.shutil.copy.assert_not_called()


def test_unknown_variant_with_size(create_command, tmp_path, capsys):
    """If the app specifies an unknown variant with a size, but no variants are
    specified, a message is output."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original-3742.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source={
            "round": "input/original",
        },
        variant="unknown",
        size="3742",
        target=out_path,
    )

    # The right message was written to output
    expected = "Unknown variant 'unknown' for 3742px sample image; using default\n"
    assert capsys.readouterr().out == expected

    # No file was installed.
    create_command.tools.shutil.copy.assert_not_called()


def test_unsized_variant(create_command, tmp_path, capsys):
    """If the app specifies an unsized variant, it is used."""
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)

    # Create the source image
    source_file = tmp_path / "base_path/input/original.png"
    source_file.parent.mkdir(parents=True, exist_ok=True)
    with source_file.open("w", encoding="utf-8") as f:
        f.write("image")

    # Try to install the image
    # Unsized variants are an annoying edge case; they get the *variant*
    # as the *size*.
    out_path = tmp_path / "output.png"
    create_command.install_image(
        "sample image",
        source={
            "round": "input/original",
        },
        variant=None,
        size="round",
        target=out_path,
    )

    # The right message was written to output
    expected = (
        "Installing input/original.png as round sample image... started\n"
        "Installing input/original.png as round sample image... done\n\n"
    )
    assert capsys.readouterr().out == expected

    # The file was copied into position
    create_command.tools.shutil.copy.assert_called_with(
        create_command.base_path / "input/original.png",
        out_path,
    )



import shutil
import subprocess
import sys
import tarfile
from pathlib import Path
from unittest import mock

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseCommandError
from briefcase.platforms.linux import system
from briefcase.platforms.linux.system import LinuxSystemPackageCommand

from ....utils import create_file, create_tgz_file


@pytest.fixture
def package_command(first_app, tmp_path):
    command = LinuxSystemPackageCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.home_path = tmp_path / "home"

    # Mock ABI from packaging system
    command._rpm_abi = "wonky"

    # Mock the app context
    command.tools.app_tools[first_app].app_context = mock.MagicMock()

    # Mock shutil
    command.tools.shutil = mock.MagicMock()

    # Make the mock make_archive still package tarballs
    command.tools.shutil.make_archive = mock.MagicMock(side_effect=shutil.make_archive)

    # Make the mock rmtree still remove content
    command.tools.shutil.rmtree = mock.MagicMock(side_effect=shutil.rmtree)

    # Mock the RPM tag, since "somevendor" won't identify as rpm.
    command.rpm_tag = mock.MagicMock(return_value="fcXX")

    # Mock not using docker
    command.target_image = None
    command.extra_docker_build_args = []

    return command


@pytest.fixture
def first_app_rpm(first_app, tmp_path):
    # Mock a Red Hat app
    first_app.python_version_tag = "3"
    first_app.target_vendor_base = "rhel"
    first_app.packaging_format = "rpm"
    first_app.glibc_version = "2.99"
    first_app.long_description = "Long description\nfor the app"

    # Mock the side effects of building the app
    usr_dir = (
        tmp_path / "base_path/build/first-app/somevendor/surprising/first-app-0.0.1/usr"
    )

    # Create the binary
    create_file(usr_dir / "bin/first-app", "binary")

    # Files in an app-named folder
    create_file(usr_dir / "share/doc/first-app/license", "license")
    create_file(usr_dir / "share/doc/first-app/UserManual", "manual")

    # A share file in an app-named folder
    create_file(usr_dir / "share/man/man1/first-app.1.gz", "man")

    return first_app


def test_verify_no_docker(package_command, first_app_rpm, monkeypatch):
    """If not using docker, existence of rpmbuild is verified."""
    # Mock the existence of rpmbuild
    package_command.tools.shutil.which = mock.MagicMock(return_value="/mybin/rpmbuild")

    # App tools can be verified
    package_command.verify_app_tools(first_app_rpm)

    # rpmbuild was inspected
    package_command.tools.shutil.which.assert_called_once_with("rpmbuild")


@pytest.mark.parametrize(
    "vendor_base, error_msg",
    [
        (
            "rhel",
            "Can't find the rpm-build tools. Try running `sudo dnf install rpm-build`.",
        ),
        (None, "Can't find the rpmbuild tool. Install this first to package the rpm."),
    ],
)
def test_verify_rpmbuild_missing(
    package_command,
    first_app_rpm,
    vendor_base,
    error_msg,
    monkeypatch,
):
    """If rpmbuild isn't installed, an error is raised."""
    # Mock distro so packager is found or not appropriately
    first_app_rpm.target_vendor_base = vendor_base

    # Mock packager as missing
    package_command.tools.shutil.which = mock.MagicMock(return_value="")

    # Verifying app tools will raise an error
    with pytest.raises(BriefcaseCommandError, match=error_msg):
        package_command.verify_app_tools(first_app_rpm)

    # rpmbuild was inspected
    package_command.tools.shutil.which.assert_called_once_with("rpmbuild")


def test_verify_docker(package_command, first_app_rpm, monkeypatch):
    """If using Docker, no tool checks are needed."""
    # Mock using docker
    package_command.target_image = "somevendor:surprising"

    # Mock the existence of a valid non-docker system Python
    # with the same major/minor as the current Python,
    # plus the existence of rpmbuild
    python3 = mock.MagicMock()
    python3.resolve.return_value = Path(
        f"/usr/bin/python{sys.version_info.major}.{sys.version_info.minor}"
    )

    rpmbuild = mock.MagicMock()
    rpmbuild.exists.return_value = False

    mock_Path = mock.MagicMock(side_effect=[python3, rpmbuild])
    monkeypatch.setattr(system, "Path", mock_Path)

    # App tools can be verified
    package_command.verify_app_tools(first_app_rpm)

    # rpmbuild was not inspected
    rpmbuild.exists.assert_not_called()


@pytest.mark.skipif(sys.platform == "win32", reason="Can't build RPMs on Windows")
def test_rpm_package(package_command, first_app_rpm, tmp_path):
    """A rpm app can be packaged."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Package the app
    package_command.package_app(first_app_rpm)

    # rpmbuild layout has been generated
    assert (bundle_path / "rpmbuild/BUILD").exists()
    assert (bundle_path / "rpmbuild/BUILDROOT").exists()
    assert (bundle_path / "rpmbuild/RPMS").exists()
    assert (bundle_path / "rpmbuild/SOURCES").exists()
    assert (bundle_path / "rpmbuild/SRPMS").exists()
    assert (bundle_path / "rpmbuild/SPECS").exists()

    # The spec file is written
    assert (bundle_path / "rpmbuild/SPECS/first-app.spec").exists()
    with (bundle_path / "rpmbuild/SPECS/first-app.spec").open(encoding="utf-8") as f:
        assert f.read() == "\n".join(
            [
                "%global __brp_mangle_shebangs %{nil}",
                "%global __brp_strip %{nil}",
                "%global __brp_strip_static_archive %{nil}",
                "%global __brp_strip_comment_note %{nil}",
                "%global __brp_check_rpaths %{nil}",
                "%global __requires_exclude_from ^%{_libdir}/first-app/.*$",
                "%global __provides_exclude_from ^%{_libdir}/first-app/.*$",
                "%global _enable_debug_package 0",
                "%global debug_package %{nil}",
                "",
                "Name:           first-app",
                "Version:        0.0.1",
                "Release:        1%{?dist}",
                "Summary:        The first simple app \\ demonstration",
                "",
                "License:        Unknown",
                "URL:            https://example.com/first-app",
                "Source0:        %{name}-%{version}.tar.gz",
                "",
                "Requires:       python3",
                "",
                "ExclusiveArch:  wonky",
                "",
                "%description",
                "Long description",
                "for the app",
                "",
                "%prep",
                "%autosetup",
                "",
                "%build",
                "",
                "%install",
                "cp -r usr %{buildroot}/usr",
                "",
                "%files",
                '"/usr/bin/first-app"',
                '%dir "/usr/lib/first-app"',
                '%dir "/usr/lib/first-app/app"',
                '"/usr/lib/first-app/app/support.so"',
                '"/usr/lib/first-app/app/support_same_perms.so"',
                '%dir "/usr/lib/first-app/app_packages"',
                '%dir "/usr/lib/first-app/app_packages/firstlib"',
                '"/usr/lib/first-app/app_packages/firstlib/first.so"',
                '"/usr/lib/first-app/app_packages/firstlib/first.so.1.0"',
                '%dir "/usr/lib/first-app/app_packages/secondlib"',
                '"/usr/lib/first-app/app_packages/secondlib/second_a.so"',
                '"/usr/lib/first-app/app_packages/secondlib/second_b.so"',
                '%dir "/usr/share/doc/first-app"',
                '"/usr/share/doc/first-app/UserManual"',
                '"/usr/share/doc/first-app/license"',
                '"/usr/share/man/man1/first-app.1.gz"',
                "",
                "%changelog",
                "First App Changelog",
            ]
        )

    # A source tarball was created with the right content
    archive_file = bundle_path / "rpmbuild/SOURCES/first-app-0.0.1.tar.gz"
    assert archive_file.exists()
    with tarfile.open(archive_file, "r:gz") as archive:
        assert sorted(archive.getnames()) == [
            "first-app-0.0.1",
            "first-app-0.0.1/usr",
            "first-app-0.0.1/usr/bin",
            "first-app-0.0.1/usr/bin/first-app",
            "first-app-0.0.1/usr/lib",
            "first-app-0.0.1/usr/lib/first-app",
            "first-app-0.0.1/usr/lib/first-app/app",
            "first-app-0.0.1/usr/lib/first-app/app/support.so",
            "first-app-0.0.1/usr/lib/first-app/app/support_same_perms.so",
            "first-app-0.0.1/usr/lib/first-app/app_packages",
            "first-app-0.0.1/usr/lib/first-app/app_packages/firstlib",
            "first-app-0.0.1/usr/lib/first-app/app_packages/firstlib/first.so",
            "first-app-0.0.1/usr/lib/first-app/app_packages/firstlib/first.so.1.0",
            "first-app-0.0.1/usr/lib/first-app/app_packages/secondlib",
            "first-app-0.0.1/usr/lib/first-app/app_packages/secondlib/second_a.so",
            "first-app-0.0.1/usr/lib/first-app/app_packages/secondlib/second_b.so",
            "first-app-0.0.1/usr/share",
            "first-app-0.0.1/usr/share/doc",
            "first-app-0.0.1/usr/share/doc/first-app",
            "first-app-0.0.1/usr/share/doc/first-app/UserManual",
            "first-app-0.0.1/usr/share/doc/first-app/license",
            "first-app-0.0.1/usr/share/man",
            "first-app-0.0.1/usr/share/man/man1",
            "first-app-0.0.1/usr/share/man/man1/first-app.1.gz",
        ]

    # rpmbuild was invoked
    package_command.tools.app_tools[
        first_app_rpm
    ].app_context.run.assert_called_once_with(
        [
            "rpmbuild",
            "-bb",
            "--define",
            f"_topdir {bundle_path / 'rpmbuild'}",
            "./rpmbuild/SPECS/first-app.spec",
        ],
        check=True,
        cwd=bundle_path,
    )

    # The rpm was moved into the final location
    package_command.tools.shutil.move.assert_called_once_with(
        bundle_path
        / "rpmbuild"
        / "RPMS"
        / "wonky"
        / "first-app-0.0.1-1.fcXX.wonky.rpm",
        tmp_path / "base_path/dist/first-app-0.0.1-1.fcXX.wonky.rpm",
    )


@pytest.mark.skipif(sys.platform == "win32", reason="Can't build RPMs on Windows")
def test_rpm_re_package(package_command, first_app_rpm, tmp_path):
    """A rpm app that has previously been packaged can be re-packaged."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Create an old spec file and tarball that will be overwritten.
    create_file(
        bundle_path / "rpmbuild/SPECS/first-app.spec",
        "Old spec content",
    )
    create_tgz_file(
        bundle_path / "rpmbuild/SOURCES/first-app-0.0.1.tar.gz",
        [("old.txt", "old content")],
    )

    # Package the app
    package_command.package_app(first_app_rpm)

    # rpmbuild layout has been generated
    assert (bundle_path / "rpmbuild/BUILD").exists()
    assert (bundle_path / "rpmbuild/BUILDROOT").exists()
    assert (bundle_path / "rpmbuild/RPMS").exists()
    assert (bundle_path / "rpmbuild/SOURCES").exists()
    assert (bundle_path / "rpmbuild/SRPMS").exists()
    assert (bundle_path / "rpmbuild/SPECS").exists()

    # The spec file is written
    assert (bundle_path / "rpmbuild/SPECS/first-app.spec").exists()
    with (bundle_path / "rpmbuild/SPECS/first-app.spec").open(encoding="utf-8") as f:
        assert f.read() == "\n".join(
            [
                "%global __brp_mangle_shebangs %{nil}",
                "%global __brp_strip %{nil}",
                "%global __brp_strip_static_archive %{nil}",
                "%global __brp_strip_comment_note %{nil}",
                "%global __brp_check_rpaths %{nil}",
                "%global __requires_exclude_from ^%{_libdir}/first-app/.*$",
                "%global __provides_exclude_from ^%{_libdir}/first-app/.*$",
                "%global _enable_debug_package 0",
                "%global debug_package %{nil}",
                "",
                "Name:           first-app",
                "Version:        0.0.1",
                "Release:        1%{?dist}",
                "Summary:        The first simple app \\ demonstration",
                "",
                "License:        Unknown",
                "URL:            https://example.com/first-app",
                "Source0:        %{name}-%{version}.tar.gz",
                "",
                "Requires:       python3",
                "",
                "ExclusiveArch:  wonky",
                "",
                "%description",
                "Long description",
                "for the app",
                "",
                "%prep",
                "%autosetup",
                "",
                "%build",
                "",
                "%install",
                "cp -r usr %{buildroot}/usr",
                "",
                "%files",
                '"/usr/bin/first-app"',
                '%dir "/usr/lib/first-app"',
                '%dir "/usr/lib/first-app/app"',
                '"/usr/lib/first-app/app/support.so"',
                '"/usr/lib/first-app/app/support_same_perms.so"',
                '%dir "/usr/lib/first-app/app_packages"',
                '%dir "/usr/lib/first-app/app_packages/firstlib"',
                '"/usr/lib/first-app/app_packages/firstlib/first.so"',
                '"/usr/lib/first-app/app_packages/firstlib/first.so.1.0"',
                '%dir "/usr/lib/first-app/app_packages/secondlib"',
                '"/usr/lib/first-app/app_packages/secondlib/second_a.so"',
                '"/usr/lib/first-app/app_packages/secondlib/second_b.so"',
                '%dir "/usr/share/doc/first-app"',
                '"/usr/share/doc/first-app/UserManual"',
                '"/usr/share/doc/first-app/license"',
                '"/usr/share/man/man1/first-app.1.gz"',
                "",
                "%changelog",
                "First App Changelog",
            ]
        )

    # A source tarball was created with the right content
    archive_file = bundle_path / "rpmbuild/SOURCES/first-app-0.0.1.tar.gz"
    assert archive_file.exists()
    with tarfile.open(archive_file, "r:gz") as archive:
        assert sorted(archive.getnames()) == [
            "first-app-0.0.1",
            "first-app-0.0.1/usr",
            "first-app-0.0.1/usr/bin",
            "first-app-0.0.1/usr/bin/first-app",
            "first-app-0.0.1/usr/lib",
            "first-app-0.0.1/usr/lib/first-app",
            "first-app-0.0.1/usr/lib/first-app/app",
            "first-app-0.0.1/usr/lib/first-app/app/support.so",
            "first-app-0.0.1/usr/lib/first-app/app/support_same_perms.so",
            "first-app-0.0.1/usr/lib/first-app/app_packages",
            "first-app-0.0.1/usr/lib/first-app/app_packages/firstlib",
            "first-app-0.0.1/usr/lib/first-app/app_packages/firstlib/first.so",
            "first-app-0.0.1/usr/lib/first-app/app_packages/firstlib/first.so.1.0",
            "first-app-0.0.1/usr/lib/first-app/app_packages/secondlib",
            "first-app-0.0.1/usr/lib/first-app/app_packages/secondlib/second_a.so",
            "first-app-0.0.1/usr/lib/first-app/app_packages/secondlib/second_b.so",
            "first-app-0.0.1/usr/share",
            "first-app-0.0.1/usr/share/doc",
            "first-app-0.0.1/usr/share/doc/first-app",
            "first-app-0.0.1/usr/share/doc/first-app/UserManual",
            "first-app-0.0.1/usr/share/doc/first-app/license",
            "first-app-0.0.1/usr/share/man",
            "first-app-0.0.1/usr/share/man/man1",
            "first-app-0.0.1/usr/share/man/man1/first-app.1.gz",
        ]

    # rpmbuild was invoked
    package_command.tools.app_tools[
        first_app_rpm
    ].app_context.run.assert_called_once_with(
        [
            "rpmbuild",
            "-bb",
            "--define",
            f"_topdir {bundle_path / 'rpmbuild'}",
            "./rpmbuild/SPECS/first-app.spec",
        ],
        check=True,
        cwd=bundle_path,
    )

    # The rpm was moved into the final location
    package_command.tools.shutil.move.assert_called_once_with(
        bundle_path / "rpmbuild/RPMS/wonky/first-app-0.0.1-1.fcXX.wonky.rpm",
        tmp_path / "base_path/dist/first-app-0.0.1-1.fcXX.wonky.rpm",
    )


@pytest.mark.skipif(sys.platform == "win32", reason="Can't build RPMs on Windows")
def test_rpm_package_no_long_description(package_command, first_app_rpm, tmp_path):
    """A rpm app without a long description raises an error."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Delete the long description
    first_app_rpm.long_description = None

    # Packaging the app will fail
    with pytest.raises(
        BriefcaseCommandError,
        match=r"App configuration does not define `long_description`. Red Hat projects require a long description.",
    ):
        package_command.package_app(first_app_rpm)

    # The spec file and tarball won't be written
    assert not (bundle_path / "rpmbuild/SPECS/first-app.spec").exists()
    assert not (bundle_path / "rpmbuild/SOURCES/first-app-0.0.1.tar.gz").exists()


@pytest.mark.skipif(sys.platform == "win32", reason="Can't build RPMs on Windows")
def test_rpm_package_extra_requirements(package_command, first_app_rpm, tmp_path):
    """A rpm app can be packaged with extra runtime requirements and config features."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Add system requirements and other optional settings.
    first_app_rpm.system_runtime_requires = ["first", "second"]
    first_app_rpm.revision = 42

    # Package the app
    package_command.package_app(first_app_rpm)

    # rpmbuild layout has been generated
    assert (bundle_path / "rpmbuild/BUILD").exists()
    assert (bundle_path / "rpmbuild/BUILDROOT").exists()
    assert (bundle_path / "rpmbuild/RPMS").exists()
    assert (bundle_path / "rpmbuild/SOURCES").exists()
    assert (bundle_path / "rpmbuild/SRPMS").exists()
    assert (bundle_path / "rpmbuild/SPECS").exists()

    # The spec file is written
    assert (bundle_path / "rpmbuild/SPECS/first-app.spec").exists()
    with (bundle_path / "rpmbuild/SPECS/first-app.spec").open(encoding="utf-8") as f:
        assert f.read() == "\n".join(
            [
                "%global __brp_mangle_shebangs %{nil}",
                "%global __brp_strip %{nil}",
                "%global __brp_strip_static_archive %{nil}",
                "%global __brp_strip_comment_note %{nil}",
                "%global __brp_check_rpaths %{nil}",
                "%global __requires_exclude_from ^%{_libdir}/first-app/.*$",
                "%global __provides_exclude_from ^%{_libdir}/first-app/.*$",
                "%global _enable_debug_package 0",
                "%global debug_package %{nil}",
                "",
                "Name:           first-app",
                "Version:        0.0.1",
                "Release:        42%{?dist}",
                "Summary:        The first simple app \\ demonstration",
                "",
                "License:        Unknown",
                "URL:            https://example.com/first-app",
                "Source0:        %{name}-%{version}.tar.gz",
                "",
                "Requires:       python3",
                "Requires:       first",
                "Requires:       second",
                "",
                "ExclusiveArch:  wonky",
                "",
                "%description",
                "Long description",
                "for the app",
                "",
                "%prep",
                "%autosetup",
                "",
                "%build",
                "",
                "%install",
                "cp -r usr %{buildroot}/usr",
                "",
                "%files",
                '"/usr/bin/first-app"',
                '%dir "/usr/lib/first-app"',
                '%dir "/usr/lib/first-app/app"',
                '"/usr/lib/first-app/app/support.so"',
                '"/usr/lib/first-app/app/support_same_perms.so"',
                '%dir "/usr/lib/first-app/app_packages"',
                '%dir "/usr/lib/first-app/app_packages/firstlib"',
                '"/usr/lib/first-app/app_packages/firstlib/first.so"',
                '"/usr/lib/first-app/app_packages/firstlib/first.so.1.0"',
                '%dir "/usr/lib/first-app/app_packages/secondlib"',
                '"/usr/lib/first-app/app_packages/secondlib/second_a.so"',
                '"/usr/lib/first-app/app_packages/secondlib/second_b.so"',
                '%dir "/usr/share/doc/first-app"',
                '"/usr/share/doc/first-app/UserManual"',
                '"/usr/share/doc/first-app/license"',
                '"/usr/share/man/man1/first-app.1.gz"',
                "",
                "%changelog",
                "First App Changelog",
            ]
        )

    # A source tarball was created
    archive_file = bundle_path / "rpmbuild/SOURCES/first-app-0.0.1.tar.gz"
    assert archive_file.exists()

    # rpmbuild was invoked
    package_command.tools.app_tools[
        first_app_rpm
    ].app_context.run.assert_called_once_with(
        [
            "rpmbuild",
            "-bb",
            "--define",
            f"_topdir {bundle_path / 'rpmbuild'}",
            "./rpmbuild/SPECS/first-app.spec",
        ],
        check=True,
        cwd=bundle_path,
    )

    # The rpm was moved into the final location
    package_command.tools.shutil.move.assert_called_once_with(
        bundle_path
        / "rpmbuild"
        / "RPMS"
        / "wonky"
        / "first-app-0.0.1-42.fcXX.wonky.rpm",
        tmp_path / "base_path/dist/first-app-0.0.1-42.fcXX.wonky.rpm",
    )


@pytest.mark.skipif(sys.platform == "win32", reason="Can't build RPMs on Windows")
def test_rpm_package_failure(package_command, first_app_rpm, tmp_path):
    """If packaging doesn't succeed, an error is raised."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Mock a packaging failure
    package_command.tools.app_tools[first_app_rpm].app_context.run.side_effect = (
        subprocess.CalledProcessError(cmd="rpmbuild ...", returncode=-1)
    )

    # Package the app; this will fail
    with pytest.raises(
        BriefcaseCommandError, match=r"Error while building .rpm package for first-app."
    ):
        package_command.package_app(first_app_rpm)

    # rpmbuild layout has been generated
    assert (bundle_path / "rpmbuild/BUILD").exists()
    assert (bundle_path / "rpmbuild/BUILDROOT").exists()
    assert (bundle_path / "rpmbuild/RPMS").exists()
    assert (bundle_path / "rpmbuild/SOURCES").exists()
    assert (bundle_path / "rpmbuild/SRPMS").exists()
    assert (bundle_path / "rpmbuild/SPECS").exists()

    # The spec file is written
    assert (bundle_path / "rpmbuild/SPECS/first-app.spec").exists()

    # A source tarball was created
    archive_file = bundle_path / "rpmbuild/SOURCES/first-app-0.0.1.tar.gz"
    assert archive_file.exists()

    # rpmbuild was invoked
    package_command.tools.app_tools[
        first_app_rpm
    ].app_context.run.assert_called_once_with(
        [
            "rpmbuild",
            "-bb",
            "--define",
            f"_topdir {bundle_path / 'rpmbuild'}",
            "./rpmbuild/SPECS/first-app.spec",
        ],
        check=True,
        cwd=bundle_path,
    )

    # The deb wasn't built, so it wasn't moved.
    package_command.tools.shutil.move.assert_not_called()


@pytest.mark.skipif(sys.platform == "win32", reason="Can't build RPMs on Windows")
def test_no_changelog(package_command, first_app_rpm, tmp_path):
    """If an packaging doesn't succeed, an error is raised."""
    bundle_path = tmp_path / "base_path/build/first-app/somevendor/surprising"

    # Remove the changelog file
    (tmp_path / "base_path/CHANGELOG").unlink()

    # Package the app; this will fail
    with pytest.raises(
        BriefcaseCommandError, match=r"Your project does not contain a CHANGELOG file."
    ):
        package_command.package_app(first_app_rpm)

    # rpmbuild layout has been generated
    assert (bundle_path / "rpmbuild/BUILD").exists()
    assert (bundle_path / "rpmbuild/BUILDROOT").exists()
    assert (bundle_path / "rpmbuild/RPMS").exists()
    assert (bundle_path / "rpmbuild/SOURCES").exists()
    assert (bundle_path / "rpmbuild/SRPMS").exists()
    assert (bundle_path / "rpmbuild/SPECS").exists()

    # The spec file will exist (however, it will be incomplete)
    assert (bundle_path / "rpmbuild/SPECS/first-app.spec").exists()

    # No source tarball was created
    archive_file = bundle_path / "rpmbuild/SOURCES/first-app-0.0.1.tar.gz"
    assert not archive_file.exists()

    # rpmbuild wasn't invoked
    package_command.tools.app_tools[first_app_rpm].app_context.run.assert_not_called()

    # The deb wasn't built, so it wasn't moved.
    package_command.tools.shutil.move.assert_not_called()



from unittest.mock import MagicMock

import pytest

from ...utils import PartialMatchString


@pytest.mark.parametrize("license_file_name", ["LICENSE", "LICENCE"])
@pytest.mark.parametrize(
    "license_text, license",
    [
        ("MIT", "MIT license"),
        ("MIT license", "MIT license"),
        ("Permission is hereby granted, free of charge", "MIT license"),
        ("Apache license", "Apache Software License"),
        ("BSD", "BSD license"),
        ("BSD license", "BSD license"),
        # Includes some extra text to ensure it doesn't get caught as MIT because of
        # perMITted
        (
            "Redistribution and use in source and binary forms, with or without modification, are permitted",
            "BSD license",
        ),
        ("GPLv2", "GNU General Public License v2 (GPLv2)"),
        (
            "version 2 of the GNU General Public License",
            "GNU General Public License v2 (GPLv2)",
        ),
        ("GPLv2+", "GNU General Public License v2 or later (GPLv2+)"),
        (
            "Free Software Foundation, either version 2 of the License",
            "GNU General Public License v2 or later (GPLv2+)",
        ),
        ("GPLv3", "GNU General Public License v3 (GPLv3)"),
        (
            "version 3 of the GNU General Public License",
            "GNU General Public License v3 (GPLv3)",
        ),
        ("GPLv3+", "GNU General Public License v3 or later (GPLv3+)"),
        (
            "either version 3 of the License",
            "GNU General Public License v3 or later (GPLv3+)",
        ),
        ("Some text", "Other"),
    ],
)
def test_get_license_from_file(
    convert_command, license_text, license, license_file_name, monkeypatch
):
    mock_select_option = MagicMock()
    monkeypatch.setattr(convert_command, "select_option", mock_select_option)

    dummy_license_text = (
        "some leading text\neven_more_text" + license_text + "some_ending_text\n"
    )
    (convert_command.base_path / license_file_name).write_text(
        dummy_license_text, encoding="utf-8"
    )

    convert_command.input_license(None)
    mock_select_option.assert_called_once_with(
        intro=PartialMatchString("the license file"),
        variable="Project License",
        options=[
            "BSD license",
            "MIT license",
            "Apache Software License",
            "GNU General Public License v2 (GPLv2)",
            "GNU General Public License v2 or later (GPLv2+)",
            "GNU General Public License v3 (GPLv3)",
            "GNU General Public License v3 or later (GPLv3+)",
            "Proprietary",
            "Other",
        ],
        default=license,
        override_value=None,
    )


@pytest.mark.parametrize(
    "license_text, license",
    [
        ("MIT", "MIT license"),
        ("MIT license", "MIT license"),
        ("Permission is hereby granted, free of charge", "MIT license"),
        ("Apache license", "Apache Software License"),
        ("BSD", "BSD license"),
        ("BSD license", "BSD license"),
        # Includes some extra text to ensure it doesn't get caught as MIT because of
        # perMITted
        (
            "Redistribution and use in source and binary forms, with or without modification, are permitted",
            "BSD license",
        ),
        ("GPLv2", "GNU General Public License v2 (GPLv2)"),
        (
            "version 2 of the GNU General Public License",
            "GNU General Public License v2 (GPLv2)",
        ),
        ("GPLv2+", "GNU General Public License v2 or later (GPLv2+)"),
        (
            "Free Software Foundation, either version 2 of the License",
            "GNU General Public License v2 or later (GPLv2+)",
        ),
        ("GPLv3", "GNU General Public License v3 (GPLv3)"),
        (
            "version 3 of the GNU General Public License",
            "GNU General Public License v3 (GPLv3)",
        ),
        ("GPLv3+", "GNU General Public License v3 or later (GPLv3+)"),
        (
            "either version 3 of the License",
            "GNU General Public License v3 or later (GPLv3+)",
        ),
        ("Some text", "Other"),
    ],
)
def test_get_license_from_pep621_license_file(
    convert_command,
    license_text,
    license,
    monkeypatch,
):
    mock_select_option = MagicMock()
    monkeypatch.setattr(convert_command, "select_option", mock_select_option)

    dummy_license_text = (
        "some leading text\neven_more_text" + license_text + "some_ending_text\n"
    )
    (convert_command.base_path / "LICENSE.txt").write_text(
        dummy_license_text, encoding="utf-8"
    )
    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n" 'license = {file = "LICENSE.txt"}', encoding="utf-8"
    )

    convert_command.input_license(None)

    mock_select_option.assert_called_once_with(
        intro=PartialMatchString("the license file"),
        variable="Project License",
        options=[
            "BSD license",
            "MIT license",
            "Apache Software License",
            "GNU General Public License v2 (GPLv2)",
            "GNU General Public License v2 or later (GPLv2+)",
            "GNU General Public License v3 (GPLv3)",
            "GNU General Public License v3 or later (GPLv3+)",
            "Proprietary",
            "Other",
        ],
        default=license,
        override_value=None,
    )


@pytest.mark.parametrize(
    "license_text, license",
    [
        ("MIT", "MIT license"),
        ("MIT license", "MIT license"),
        ("BSD", "BSD license"),
        ("BSD license", "BSD license"),
        ("GPLv2", "GNU General Public License v2 (GPLv2)"),
        ("GPLv2+", "GNU General Public License v2 or later (GPLv2+)"),
        ("GPLv3", "GNU General Public License v3 (GPLv3)"),
        ("GPLv3+", "GNU General Public License v3 or later (GPLv3+)"),
        ("Some text", "Other"),
    ],
)
def test_get_license_from_pyproject(
    convert_command,
    license_text,
    license,
    monkeypatch,
):
    mock_select_option = MagicMock()
    monkeypatch.setattr(convert_command, "select_option", mock_select_option)
    (convert_command.base_path / "pyproject.toml").write_text(
        "[project]\n" f'license = {{text = "{license_text}"}}', encoding="utf-8"
    )

    convert_command.input_license(None)
    mock_select_option.assert_called_once_with(
        intro=PartialMatchString("the PEP621 formatted pyproject.toml"),
        variable="Project License",
        options=[
            "BSD license",
            "MIT license",
            "Apache Software License",
            "GNU General Public License v2 (GPLv2)",
            "GNU General Public License v2 or later (GPLv2+)",
            "GNU General Public License v3 (GPLv3)",
            "GNU General Public License v3 or later (GPLv3+)",
            "Proprietary",
            "Other",
        ],
        default=license,
        override_value=None,
    )


def test_no_license_hint(convert_command, monkeypatch):
    mock_select_option = MagicMock()
    monkeypatch.setattr(convert_command, "select_option", mock_select_option)

    convert_command.input_license(None)
    mock_select_option.assert_called_once_with(
        intro="What license do you want to use for this project's code? ",
        variable="Project License",
        options=[
            "BSD license",
            "MIT license",
            "Apache Software License",
            "GNU General Public License v2 (GPLv2)",
            "GNU General Public License v2 or later (GPLv2+)",
            "GNU General Public License v3 (GPLv3)",
            "GNU General Public License v3 or later (GPLv3+)",
            "Proprietary",
            "Other",
        ],
        default=None,
        override_value=None,
    )


def test_override_is_used(convert_command):
    assert convert_command.input_license("Proprietary") == "Proprietary"



import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import UnsupportedHostError
from briefcase.platforms.macOS.app import macOSAppCreateCommand, macOSAppPackageCommand


@pytest.fixture
def create_command(tmp_path):
    return macOSAppCreateCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )


@pytest.fixture
def package_command(tmp_path):
    return macOSAppPackageCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )


@pytest.mark.parametrize("host_os", ["Linux", "Windows", "WeirdOS"])
def test_unsupported_host_os(create_command, host_os):
    """Error raised for an unsupported OS."""
    create_command.tools.host_os = host_os

    with pytest.raises(
        UnsupportedHostError,
        match="macOS applications can only be built on macOS.",
    ):
        create_command()


def test_binary_path(create_command, first_app_config, tmp_path):
    binary_path = create_command.binary_path(first_app_config)

    expected_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "app"
        / "First App.app"
    )
    assert binary_path == expected_path


def test_project_path(create_command, first_app_config, tmp_path):
    """The project path is the Contents directory."""
    project_path = create_command.project_path(first_app_config)

    expected_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "app"
        / "First App.app"
        / "Contents"
    )
    assert expected_path == project_path


def test_distribution_path_zip(package_command, first_app_config, tmp_path):
    first_app_config.packaging_format = "zip"
    distribution_path = package_command.distribution_path(first_app_config)

    expected_path = tmp_path / "base_path/dist/First App-0.0.1.app.zip"
    assert distribution_path == expected_path


def test_distribution_path_dmg(package_command, first_app_config, tmp_path):
    first_app_config.packaging_format = "dmg"
    distribution_path = package_command.distribution_path(first_app_config)

    expected_path = tmp_path / "base_path/dist/First App-0.0.1.dmg"
    assert distribution_path == expected_path


def test_distribution_path_pkg(package_command, first_app_config, tmp_path):
    first_app_config.packaging_format = "pkg"
    distribution_path = package_command.distribution_path(first_app_config)

    expected_path = tmp_path / "base_path/dist/First App-0.0.1.pkg"
    assert distribution_path == expected_path



import pytest

from briefcase.commands import PublishCommand
from briefcase.commands.base import full_options
from briefcase.config import AppConfig
from briefcase.console import Console, Log

from ...utils import create_file


class DummyPublishCommand(PublishCommand):
    """A dummy publish command that doesn't actually do anything.

    It only serves to track which actions would be performed.
    """

    # Platform and format contain upper case to test case normalization
    platform = "Tester"
    output_format = "Dummy"
    description = "Dummy publish command"

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("logger", Log())
        kwargs.setdefault("console", Console())
        super().__init__(*args, apps={}, **kwargs)

        self.actions = []

    def briefcase_toml(self, app):
        # default any app to an empty `briefcase.toml`
        return self._briefcase_toml.get(app, {})

    def binary_path(self, app):
        return self.bundle_path(app) / f"{app.app_name}.bin"

    def verify_host(self):
        super().verify_host()
        self.actions.append(("verify-host",))

    def verify_tools(self):
        super().verify_tools()
        self.actions.append(("verify-tools",))

    def finalize_app_config(self, app):
        super().finalize_app_config(app=app)
        self.actions.append(("finalize-app-config", app.app_name))

    def verify_app_template(self, app):
        super().verify_app_template(app=app)
        self.actions.append(("verify-app-template", app.app_name))

    def verify_app_tools(self, app):
        super().verify_app_tools(app=app)
        self.actions.append(("verify-app-tools", app.app_name))

    @property
    def publication_channels(self):
        return ["s3", "alternative"]

    @property
    def default_publication_channel(self):
        return "s3"

    def publish_app(self, app, channel, **kwargs):
        self.actions.append(("publish", app.app_name, channel, kwargs.copy()))
        return full_options({"publish_state": app.app_name}, kwargs)

    # These commands override the default behavior, simply tracking that
    # they were invoked, rather than instantiating a Create/Update/Build command.
    # This is for testing purposes.
    def create_command(self, app, **kwargs):
        self.actions.append(("create", app.app_name, kwargs.copy()))
        # Remove arguments consumed by the underlying call to create_app()
        kwargs.pop("test_mode", None)
        return full_options({"create_state": app.app_name}, kwargs)

    def update_command(self, app, **kwargs):
        self.actions.append(("update", app.app_name, kwargs.copy()))
        # Remove arguments consumed by the underlying call to update_app()
        kwargs.pop("test_mode", None)
        kwargs.pop("update_requirements", None)
        kwargs.pop("update_resources", None)
        kwargs.pop("update_support", None)
        return full_options({"update_state": app.app_name}, kwargs)

    def build_command(self, app, **kwargs):
        self.actions.append(("build", app.app_name, kwargs.copy()))
        # Remove arguments consumed by the underlying call to build_app()
        kwargs.pop("test_mode", None)
        kwargs.pop("update", None)
        return full_options({"build_state": app.app_name}, kwargs)


@pytest.fixture
def publish_command(tmp_path):
    return DummyPublishCommand(base_path=tmp_path / "base_path")


@pytest.fixture
def first_app_config():
    return AppConfig(
        app_name="first",
        bundle="com.example",
        version="0.0.1",
        description="The first simple app",
        sources=["src/first"],
        license={"file": "LICENSE"},
    )


@pytest.fixture
def first_app_unbuilt(first_app_config, tmp_path):
    # The same fixture as first_app_config,
    # but ensures that the bundle for the app exists
    create_file(
        tmp_path
        / "base_path"
        / "build"
        / "tester"
        / "first"
        / "tester"
        / "dummy"
        / "first.bundle",
        "first.bundle",
    )

    return first_app_config


@pytest.fixture
def first_app(first_app_unbuilt, tmp_path):
    # The same fixture as first_app_config,
    # but ensures that the binary for the app exists
    create_file(
        tmp_path / "base_path/build/first/tester/dummy/first.bin",
        "first.bin",
    )

    return first_app_unbuilt


@pytest.fixture
def second_app_config():
    return AppConfig(
        app_name="second",
        bundle="com.example",
        version="0.0.2",
        description="The second simple app",
        sources=["src/second"],
        license={"file": "LICENSE"},
    )


@pytest.fixture
def second_app(second_app_config, tmp_path):
    # The same fixture as second_app_config,
    # but ensures that the binary for the app exists
    create_file(
        tmp_path
        / "base_path"
        / "build"
        / "second"
        / "tester"
        / "dummy"
        / "second.bundle",
        "second.bundle",
    )
    create_file(
        tmp_path / "base_path/build/second/tester/dummy/second.bin",
        "second.bin",
    )

    return second_app_config



from unittest.mock import MagicMock

import pytest

import briefcase.console


def test_wait_bar_done_message_interactive(console, capsys):
    """Custom done_message is printed when wait bar normally exits."""
    with console.wait_bar("Wait message...", done_message="finished"):
        pass

    assert capsys.readouterr().out == "Wait message... finished\n\n"


def test_wait_bar_done_message_non_interactive(non_interactive_console, capsys):
    """Custom done_message is printed when wait bar normally exits."""
    with non_interactive_console.wait_bar("Wait message...", done_message="finished"):
        pass

    assert capsys.readouterr().out == (
        "Wait message... started\n" "Wait message... finished\n\n"
    )


def test_wait_bar_done_message_nested_interactive(console, capsys):
    """Custom done_message is printed when wait bar normally exits."""
    with console.wait_bar("Wait message 1...", done_message="finished"):
        with console.wait_bar("Wait message 2...", done_message="finished"):
            pass

    expected = "Wait message 2... finished\n" "Wait message 1... finished\n\n"
    assert capsys.readouterr().out == expected


def test_wait_bar_done_message_nested_non_interactive(non_interactive_console, capsys):
    """Custom done_message is printed when wait bar normally exits."""
    with non_interactive_console.wait_bar("Wait message 1...", done_message="finished"):
        with non_interactive_console.wait_bar(
            "Wait message 2...", done_message="finished"
        ):
            pass

    expected = (
        "Wait message 1... started\n"
        "Wait message 2... started\n"
        "Wait message 2... finished\n"
        "Wait message 1... finished\n\n"
    )
    assert capsys.readouterr().out == expected


@pytest.mark.parametrize(
    ("message", "transient", "output"),
    (
        ("Wait message...", False, "Wait message... done\n\n"),
        ("", False, "\n"),
        ("Wait Message...", True, "\n"),
        ("", True, "\n"),
    ),
)
def test_wait_bar_transient_interactive(console, message, transient, output, capsys):
    """Output is present or absent based on presence of message and transient value."""
    with console.wait_bar(message, transient=transient):
        pass

    assert capsys.readouterr().out == output


@pytest.mark.parametrize(
    ("message", "transient", "output"),
    (
        ("Wait message...", False, "Wait message... started\nWait message... done\n\n"),
        ("", False, "\n"),
        ("Wait message...", True, "Wait message... started\nWait message... done\n\n"),
        ("", True, "\n"),
    ),
)
def test_wait_bar_transient_non_interactive(
    non_interactive_console,
    message,
    transient,
    output,
    capsys,
):
    """Output is present or absent based on presence of message and transient value."""
    with non_interactive_console.wait_bar(message, transient=transient):
        pass

    assert capsys.readouterr().out == output


@pytest.mark.parametrize(
    ("message_one", "message_two", "transient", "output"),
    (
        (
            "Wait message 1...",
            "Wait message 2...",
            False,
            "Wait message 2... done\nWait message 1... done\n\n",
        ),
        ("", "", False, "\n"),
        ("Wait message 1...", "Wait message 2...", True, "\n"),
        ("", "", True, "\n"),
    ),
)
def test_wait_bar_transient_nested_interactive(
    console,
    message_one,
    message_two,
    transient,
    output,
    capsys,
):
    """Output is present or absent based on presence of message and transient value."""
    with console.wait_bar(message_one, transient=transient):
        with console.wait_bar(message_two, transient=transient):
            pass

    assert capsys.readouterr().out == output


@pytest.mark.parametrize(
    ("message_one", "message_two", "transient", "output"),
    (
        (
            "Wait message 1...",
            "Wait message 2...",
            False,
            (
                "Wait message 1... started\n"
                "Wait message 2... started\n"
                "Wait message 2... done\n"
                "Wait message 1... done\n\n"
            ),
        ),
        ("", "", False, "\n"),
        (
            "Wait message 1...",
            "Wait message 2...",
            True,
            (
                "Wait message 1... started\n"
                "Wait message 2... started\n"
                "Wait message 2... done\n"
                "Wait message 1... done\n\n"
            ),
        ),
        ("", "", True, "\n"),
    ),
)
def test_wait_bar_transient_nested_non_interactive(
    non_interactive_console,
    message_one,
    message_two,
    transient,
    output,
    capsys,
):
    """Output is present or absent based on presence of message and transient value."""
    with non_interactive_console.wait_bar(message_one, transient=transient):
        with non_interactive_console.wait_bar(message_two, transient=transient):
            pass

    assert capsys.readouterr().out == output


@pytest.mark.parametrize(
    ("message", "transient", "output"),
    (
        ("Wait message...", False, "Wait message... aborted\n\n"),
        ("", False, "\n"),
        ("Wait Message...", True, "\n"),
        ("", True, "\n"),
    ),
)
def test_wait_bar_keyboard_interrupt_interactive(
    console,
    message,
    transient,
    output,
    capsys,
):
    """If the wait bar is interrupted, output is present or absent based on presence of
    message and transient value."""
    with pytest.raises(KeyboardInterrupt):
        with console.wait_bar(message, transient=transient):
            raise KeyboardInterrupt

    assert capsys.readouterr().out == output


@pytest.mark.parametrize(
    ("message", "transient", "output"),
    (
        (
            "Wait message...",
            False,
            "Wait message... started\nWait message... aborted\n\n",
        ),
        ("", False, "\n"),
        (
            "Wait message...",
            True,
            "Wait message... started\nWait message... aborted\n\n",
        ),
        ("", True, "\n"),
    ),
)
def test_wait_bar_keyboard_interrupt_non_interactive(
    non_interactive_console,
    message,
    transient,
    output,
    capsys,
):
    """If the wait bar is interrupted, output is present or absent based on presence of
    message and transient value."""
    with pytest.raises(KeyboardInterrupt):
        with non_interactive_console.wait_bar(message, transient=transient):
            raise KeyboardInterrupt

    assert capsys.readouterr().out == output


@pytest.mark.parametrize(
    ("message_one", "message_two", "transient", "output"),
    (
        (
            "Wait message 1...",
            "Wait message 2...",
            False,
            "Wait message 2... aborted\nWait message 1... aborted\n\n",
        ),
        ("", "", False, "\n"),
        ("Wait message 1...", "Wait message 2...", True, "\n"),
        ("", "", True, "\n"),
    ),
)
def test_wait_bar_keyboard_interrupt_nested_interactive(
    console,
    message_one,
    message_two,
    transient,
    output,
    capsys,
):
    """If the wait bar is interrupted, output is present or absent based on presence of
    message and transient value."""
    with pytest.raises(KeyboardInterrupt):
        with console.wait_bar(message_one, transient=transient):
            with console.wait_bar(message_two, transient=transient):
                raise KeyboardInterrupt

    assert capsys.readouterr().out == output


@pytest.mark.parametrize(
    ("message_one", "message_two", "transient", "output"),
    (
        (
            "Wait message 1...",
            "Wait message 2...",
            False,
            (
                "Wait message 1... started\n"
                "Wait message 2... started\n"
                "Wait message 2... aborted\n"
                "Wait message 1... aborted\n\n"
            ),
        ),
        ("", "", False, "\n"),
        (
            "Wait message 1...",
            "Wait message 2...",
            True,
            (
                "Wait message 1... started\n"
                "Wait message 2... started\n"
                "Wait message 2... aborted\n"
                "Wait message 1... aborted\n\n"
            ),
        ),
        ("", "", True, "\n"),
    ),
)
def test_wait_bar_keyboard_interrupt_nested_non_interactive(
    non_interactive_console,
    message_one,
    message_two,
    transient,
    output,
    capsys,
):
    """If the wait bar is interrupted, output is present or absent based on presence of
    message and transient value."""
    with pytest.raises(KeyboardInterrupt):
        with non_interactive_console.wait_bar(message_one, transient=transient):
            with non_interactive_console.wait_bar(message_two, transient=transient):
                raise KeyboardInterrupt

    assert capsys.readouterr().out == output


def test_wait_bar_always_interactive(console):
    """Wait Bar is not disabled when console is interactive."""
    with console.wait_bar():
        assert console._wait_bar.disable is False
        with console.wait_bar():
            assert console._wait_bar.disable is False


def test_wait_bar_non_interactive(non_interactive_console):
    """Wait Bar is disabled when console is non-interactive."""
    with non_interactive_console.wait_bar():
        assert non_interactive_console._wait_bar.disable is True
        with non_interactive_console.wait_bar():
            assert non_interactive_console._wait_bar.disable is True


def test_wait_bar_alive_messages_interactive(
    console,
    non_interactive_console,
    capsys,
    monkeypatch,
):
    """Wait Bar keep_alive prints keep alive messages."""
    for test_console in [console, non_interactive_console]:

        # initialization will set interval to a small number
        # update() will see time at a large number and print the message
        # then interval is reset back to a small number
        monkeypatch.setattr(
            briefcase.console.time,
            "time",
            MagicMock(side_effect=[0] + [1e42, 0] * 2),
        )

        with test_console.wait_bar("task") as keep_alive:
            for _ in range(2):
                keep_alive.update()

        assert capsys.readouterr().out.endswith(
            "... still waiting\n... still waiting\ntask done\n\n"
        )
        assert capsys.readouterr().out == ""



from __future__ import annotations

import subprocess
from pathlib import Path
from typing import Iterator

# winreg can only be imported on Windows
try:
    import winreg
except ImportError:  # pragma: no-cover-if-is-windows
    winreg = None

from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.base import Tool, ToolCache


class WindowsSDK(Tool):
    name = "windows_sdk"
    full_name = "Windows SDK"
    supported_host_os = {"Windows"}

    SDK_VERSION = "10.0"
    # Oldest supported SDK version is 10.0.15063.0
    SDK_MIN_VERSION = 15063
    # Registry path to keys with details of the installed Windows SDKs
    SDK_KEY = rf"SOFTWARE\Microsoft\Microsoft SDKs\Windows\v{SDK_VERSION}"
    # Subkey containing the installation directory for the SDK
    SDK_DIR_KEY = "InstallationFolder"
    # Subkey for "latest" installed SDK version
    SDK_VERSION_KEY = "ProductVersion"
    # As a fallback, possible default locations for SDK
    DEFAULT_SDK_DIRS = [
        Path(rf"C:\Program Files (x86)\Windows Kits\{SDK_VERSION.split('.')[0]}")
    ]
    # Installing parts of the SDK for UWP apps is not inherently required; however,
    # it is the minimum selection that ensures the signing tool has the libraries
    # necessary to run. Executables from older versions of the SDK may not be
    # compatible with the version of Windows that's installed.
    SDK_REQUIRED_COMPONENTS = """
    * Windows SDK Signing Tools for Desktop Apps
    * Windows SDK for UWP Managed Apps
"""

    def __init__(self, tools: ToolCache, root_path: Path, version: str, arch: str):
        """Create a wrapper around the Windows SDK.

        :param tools: ToolCache of available tools
        :param root_path: Base path for sdk, e.g. C:/Program Files (x86)/Windows
            Kits/10/
        :param version: Full SDK version, e.g. 10.0.22621.0
        :param arch: Host architecture for SDK, e.g. x64, arm64, etc
        """
        super().__init__(tools=tools)
        self.root_path = root_path
        self.version = version
        self.arch = arch

    @property
    def bin_path(self) -> Path:
        return self.root_path / "bin" / self.version / self.arch

    @property
    def signtool_exe(self) -> Path:
        return self.bin_path / "signtool.exe"

    @classmethod
    def _sdk_versions_from_bin(cls, sdk_dir: Path) -> list[str]:
        """Returns list of SDK versions from the install location bin directory."""
        bin_dir = sdk_dir / "bin"
        # prioritize newer versions of the SDK
        version_dirs = sorted(bin_dir.glob(f"{cls.SDK_VERSION}.*.0/"), reverse=True)
        return [d.name for d in version_dirs]

    @classmethod
    def _windows_sdks(cls, tools: ToolCache) -> Iterator[tuple[Path, str]]:
        """Generator of (path, version) for instances of Windows SDK.

        All instances of Windows SDK should reside in the same base directory; this is
        enforced by the SDK installer. Certain subdirectories, such as `include` and
        `bin`, will contain subdirectories for versions that may be installed.
        """
        tools.logger.debug("Finding Suitable Installation...", prefix=cls.full_name)

        # Return user-specified SDK
        if (environ_sdk_dir := tools.os.environ.get("WindowsSDKDir")) and (
            environ_sdk_version := tools.os.environ.get("WindowsSDKVersion")
        ):
            yield Path(environ_sdk_dir), environ_sdk_version
            raise BriefcaseCommandError(
                f"""\
The 'WindowsSDKDir' and 'WindowsSDKVersion' environment variables do not point
to a valid install of the Windows SDK v{cls.SDK_VERSION}:

WindowsSDKDir:     {environ_sdk_dir}
WindowsSDKVersion: {environ_sdk_version}
"""
            )

        # To support the varied bitness of processes and installations within Windows,
        # the registry is split among different views to avoid a process naively and
        # likely unintentionally referencing incompatible software/settings. This is
        # accomplished with subkeys under the primary trees, for example:
        #   HKEY_LOCAL_MACHINE\SOFTWARE\WOW6432Node
        # A 32-bit version of the registry tree is effectively mirrored under this
        # subkey for a 64-bit Windows installation.
        access_right_precedence = [
            # 32-bit process sees 32-bit registry; 64-bit process sees 64-bit registry
            winreg.KEY_READ,
            # 32-bit process sees 32-bit registry; 64-bit process sees 32-bit registry
            winreg.KEY_READ | winreg.KEY_WOW64_32KEY,
            # 32-bit process sees 64-bit registry; 64-bit process sees 64-bit registry
            winreg.KEY_READ | winreg.KEY_WOW64_64KEY,
        ]

        registry_tree_order = (
            (hkey, access)
            for hkey in [winreg.HKEY_LOCAL_MACHINE, winreg.HKEY_CURRENT_USER]
            for access in access_right_precedence
        )

        for hkey, access in registry_tree_order:
            try:
                with winreg.OpenKeyEx(hkey, cls.SDK_KEY, access=access) as key:
                    if not (sdk_dir := winreg.QueryValueEx(key, cls.SDK_DIR_KEY)[0]):
                        continue
                    if not (sdk_dir := Path(tools.os.fsdecode(sdk_dir))).is_dir():
                        continue

                    # Return the "latest" installed SDK first
                    if reg_version := winreg.QueryValueEx(key, cls.SDK_VERSION_KEY)[0]:
                        # Append missing "servicing" revision to registry version
                        reg_version = f"{reg_version}.0"
                        tools.logger.debug(
                            f"Evaluating Registry SDK version '{reg_version}' at {sdk_dir}"
                        )
                        yield sdk_dir, reg_version

                    # Return other versions of the SDK installed in sdk_dir
                    for sdk_version in cls._sdk_versions_from_bin(sdk_dir):
                        if sdk_version != reg_version:
                            tools.logger.debug(
                                f"Evaluating Registry SDK Bin version '{sdk_version}' at {sdk_dir}"
                            )
                            yield sdk_dir, sdk_version
            except FileNotFoundError:
                pass  # ignore missing registry keys

        for sdk_dir in cls.DEFAULT_SDK_DIRS:
            if sdk_dir.is_dir():
                for sdk_version in cls._sdk_versions_from_bin(sdk_dir):
                    tools.logger.debug(
                        f"Evaluating Default Bin SDK version '{sdk_version}' at {sdk_dir}"
                    )
                    yield sdk_dir, sdk_version

    @classmethod
    def _is_supported_version(cls, sdk: WindowsSDK) -> bool:
        """Returns whether the version of the SDK install is supported."""
        try:
            version_split = sdk.version.split(".")
            if not sdk.version.startswith(cls.SDK_VERSION):
                return False
            if int(version_split[2]) < cls.SDK_MIN_VERSION:
                return False
        except (AttributeError, ValueError, IndexError):
            return False

        return True

    @classmethod
    def _verify_signtool(cls, sdk: WindowsSDK) -> bool:
        """Returns SDK if signtool exists and can successfully run."""
        if not sdk.signtool_exe.is_file():
            return False

        try:
            sdk.tools.subprocess.check_output([sdk.signtool_exe, "-?"])
        except (OSError, subprocess.CalledProcessError):
            # Windows can raise OSError when it cannot run signtool. This can happen
            # when an old version of the SDK is installed and only signtool is installed.
            return False

        return True

    @classmethod
    def verify_install(cls, tools: ToolCache, **kwargs) -> WindowsSDK:
        """Verify the Windows SDK is installed with needed components.

        :param tools: ToolCache of available tools
        """
        # short circuit since already verified and available
        if hasattr(tools, "windows_sdk"):
            return tools.windows_sdk

        arch = {"AMD64": "x64", "ARM64": "arm64"}.get(tools.host_arch, tools.host_arch)

        sdk = None
        for sdk_dir, sdk_version in cls._windows_sdks(tools=tools):
            sdk = WindowsSDK(
                tools=tools,
                root_path=sdk_dir,
                version=sdk_version,
                arch=arch,
            )

            if not cls._is_supported_version(sdk):
                sdk = None
                continue

            if not cls._verify_signtool(sdk):
                sdk = None
                continue

            break

        if sdk is None:
            raise BriefcaseCommandError(
                f"""\
Unable to locate a suitable Windows SDK v{cls.SDK_VERSION} installation.

Ensure at least v{cls.SDK_VERSION}.{cls.SDK_MIN_VERSION}.0 is installed and the components below are included:
{cls.SDK_REQUIRED_COMPONENTS}
See https://developer.microsoft.com/en-us/windows/downloads/windows-sdk/ to install the SDK.
"""
            )

        tools.logger.debug(f"Using Windows SDK v{sdk.version} at {sdk.root_path}")
        tools.windows_sdk = sdk
        return sdk



import subprocess
import sys
from pathlib import Path
from unittest.mock import MagicMock

import pytest

from briefcase.exceptions import BriefcaseCommandError, UnsupportedHostError
from briefcase.integrations.docker import Docker, DockerAppContext


@pytest.fixture
def verify_kwargs():
    return dict(
        image_tag="com.example.first-app:py3.X",
        dockerfile_path=Path("/path/to/Dockerfile"),
        app_base_path=Path("/app/base"),
        host_bundle_path=Path("/host/bundle"),
        host_data_path=Path("/host/data"),
        python_version="py3.X",
        extra_build_args=["--option-one", "--option-two"],
    )


def test_short_circuit(mock_tools, first_app_config, verify_kwargs):
    """Tool is not created if already cached."""
    mock_tools[first_app_config].app_context = "tool"

    app_context = DockerAppContext.verify(mock_tools, first_app_config, **verify_kwargs)

    assert app_context == "tool"
    assert app_context == mock_tools[first_app_config].app_context


def test_unsupported_os(mock_tools, first_app_config, verify_kwargs):
    """When host OS is not supported, an error is raised."""
    mock_tools.host_os = "wonky"

    with pytest.raises(
        UnsupportedHostError,
        match=f"{DockerAppContext.name} is not supported on wonky",
    ):
        DockerAppContext.verify(mock_tools, first_app_config, **verify_kwargs)


@pytest.mark.skipif(
    sys.platform == "win32", reason="Windows paths aren't converted in Docker context"
)
@pytest.mark.usefixtures("mock_docker")
def test_success(mock_tools, first_app_config, verify_kwargs, sub_stream_kw):
    """Docker app context is successfully created and prepared."""

    DockerAppContext.verify(mock_tools, first_app_config, **verify_kwargs)

    assert isinstance(mock_tools[first_app_config].app_context, DockerAppContext)

    # Docker image is created/updated
    mock_tools.subprocess._subprocess.Popen.assert_called_with(
        [
            "docker",
            "buildx",
            "build",
            "--load",
            "--progress",
            "plain",
            "--tag",
            "com.example.first-app:py3.X",
            "--file",
            "/path/to/Dockerfile",
            "--build-arg",
            "SYSTEM_REQUIRES=",
            "--build-arg",
            "HOST_UID=37",
            "--build-arg",
            "HOST_GID=42",
            "/app/base/src",
            "--option-one",
            "--option-two",
        ],
        **sub_stream_kw,
    )


def test_docker_verify_fail(mock_tools, first_app_config, verify_kwargs, monkeypatch):
    """Failure if Docker cannot be verified."""
    monkeypatch.setattr(
        Docker,
        "verify_install",
        MagicMock(
            spec_set=Docker.verify_install,
            side_effect=BriefcaseCommandError("No docker for you"),
        ),
    )

    with pytest.raises(BriefcaseCommandError, match="No docker for you"):
        DockerAppContext.verify(mock_tools, first_app_config, **verify_kwargs)


@pytest.mark.usefixtures("mock_docker")
def test_docker_image_build_fail(mock_tools, first_app_config, verify_kwargs):
    """Failure if Docker image build fails."""
    mock_tools.subprocess._subprocess.Popen.side_effect = subprocess.CalledProcessError(
        returncode=80, cmd=["docker" "build"]
    )

    with pytest.raises(
        BriefcaseCommandError,
        match="Error building Docker container image for first-app",
    ):
        DockerAppContext.verify(mock_tools, first_app_config, **verify_kwargs)



import pytest


def test_valid_source_dir(convert_command):
    """Valid source_dir raises no errors and returns True."""
    module_name = "myapplication"
    source_dir = f"src/{module_name}"

    full_source_dir = convert_command.base_path / source_dir
    full_source_dir.mkdir(parents=True)
    (full_source_dir / "__main__.py").write_text("", encoding="utf-8")

    assert convert_command.validate_source_dir(module_name, source_dir)


def test_wrong_source_dir_name(convert_command):
    """source_dir with wrong name raises ValueError."""
    module_name = "myapplication"
    source_dir = f"src/{module_name}"

    full_source_dir = convert_command.base_path / source_dir
    full_source_dir.mkdir(parents=True)

    with pytest.raises(ValueError):
        convert_command.validate_source_dir("wrong_name", source_dir)


def test_source_dir_no_entry_point(convert_command):
    """source_dir without __main__.py raises ValueError."""
    module_name = "myapplication"
    source_dir = f"src/{module_name}"

    (convert_command.base_path / source_dir).mkdir(parents=True)
    with pytest.raises(ValueError):
        convert_command.validate_source_dir(module_name, source_dir)


def test_source_dir_does_not_exist(convert_command):
    """source_dir raises value error if {target}/{source_dir} doesn't exist."""
    module_name = "myapplication"
    source_dir = f"src/{module_name}"

    with pytest.raises(ValueError):
        convert_command.validate_source_dir(module_name, source_dir)



import pytest

from briefcase.console import Console


@pytest.mark.parametrize(
    "in_text, out_text",
    [
        (
            "There is nothing wrong with your television set.",
            "There is nothing wrong with your television set.",
        ),
        (
            "There is nothing wrong with your television set.\n"
            "Do not attempt to adjust the picture.",
            "There is nothing wrong with your television set.\n"
            "Do not attempt to adjust the picture.",
        ),
        (
            "There is nothing\n\n\nwrong with your television set.\n\n"
            "Do not attempt to adjust the picture. We are controlling transmission. If we wish to make it louder, "
            "we will bring\nup the volume.\n",
            "There is nothing\n"
            "\n"
            "\n"
            "wrong with your television set.\n"
            "\n"
            "Do not attempt to adjust the picture. We are controlling transmission. If we\n"
            "wish to make it louder, we will bring\n"
            "up the volume.",
        ),
        (
            "There is nothing wrong with your television set. Do not "
            "attempt to adjust the picture. We are controlling transmission. "
            "If we wish to make it louder, we will bring up the volume. If "
            "we wish to make it softer, we will tune it to "
            "a whisper. We will control the horizontal. We will control the vertical. "
            "We can roll the image, make it flutter. We can change the "
            "focus to a soft blur or sharpen it to crystal clarity. For the next "
            "hour, sit quietly, and we will control all that you see and hear. We repeat: There is nothing "
            "wrong with your television set.",
            "There is nothing wrong with your television set. Do not attempt to adjust the\n"
            "picture. We are controlling transmission. If we wish to make it louder, we\n"
            "will bring up the volume. If we wish to make it softer, we will tune it to a\n"
            "whisper. We will control the horizontal. We will control the vertical. We can\n"
            "roll the image, make it flutter. We can change the focus to a soft blur or\n"
            "sharpen it to crystal clarity. For the next hour, sit quietly, and we will\n"
            "control all that you see and hear. We repeat: There is nothing wrong with your\n"
            "television set.",
        ),
    ],
)
def test_textwrap(in_text, out_text):
    """Text is formatted as expected."""
    assert Console().textwrap(in_text) == out_text


@pytest.mark.parametrize(
    "width, in_text, out_text",
    [
        (20, "This is 27 characters long.", "This is 27\ncharacters long."),
        (
            50,
            "This is 57 characters long. This is 57 characters long.",
            "This is 57 characters long. This is 57 characters\nlong.",
        ),
        (
            80,
            "This is 83 characters long. This is 83 characters long. This is 83 characters long.",
            "This is 83 characters long. This is 83 characters long. This is 83 characters\nlong.",
        ),
        (
            120,
            "This is 144 characters long. This is 144 characters long. This is 144 characters long. "
            "This is 144 characters long. This is 144 characters long.",
            "This is 144 characters long. This is 144 characters long. "
            "This is 144 characters long. This is 144 characters long. This\nis 144 characters long.",
        ),
    ],
)
def test_textwrap_width_override(width, in_text, out_text):
    """Width override is respected."""
    assert Console().textwrap(in_text, width=width) == out_text



import pytest

from briefcase.console import Console, Log
from briefcase.platforms.linux.appimage import LinuxAppImagePackageCommand

from ....utils import create_file


@pytest.fixture
def package_command(tmp_path, first_app_config):
    command = LinuxAppImagePackageCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    # Mock the host architecture to something repeatable
    command.tools.host_arch = "x86_64"

    # Ensure the dist folder exists
    (tmp_path / "base_path/dist").mkdir(parents=True)

    return command


def test_package_app(package_command, first_app_config, tmp_path):
    """An AppImage can be packaged."""

    # Create the app binary
    create_file(
        tmp_path
        / "base_path/build/first-app/linux/appimage/First_App-0.0.1-x86_64.AppImage",
        "AppImage",
    )

    # Package the app
    package_command.package_app(first_app_config)

    # The binary has been copied to the dist folder
    assert (tmp_path / "base_path/dist/First_App-0.0.1-x86_64.AppImage").exists()



import sys
from unittest.mock import MagicMock

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseCommandError
from briefcase.platforms.linux import parse_freedesktop_os_release
from briefcase.platforms.linux.system import LinuxSystemRunCommand

from ....utils import create_file


def test_docker(create_command, first_app_config):
    """An app can be finalized inside docker."""
    # Build the app on a specific target
    create_command.target_image = "somevendor:surprising"
    create_command.tools.docker = MagicMock()
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response from checking /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
            "ID_LIKE=debian",
        ]
    )

    # Finalize the app config
    create_command.finalize_app_config(first_app_config)

    # The app's image, vendor and codename have been constructed from the target image
    assert first_app_config.target_image == "somevendor:surprising"
    assert first_app_config.target_vendor == "somevendor"
    assert first_app_config.target_codename == "surprising"
    assert first_app_config.target_vendor_base == "debian"

    # For tests of other properties merged in finalization, see
    # test_properties


def test_nodocker(create_command, first_app_config, tmp_path):
    """An app can be finalized without docker."""
    # Build the app without docker
    create_command.target_image = None
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    os_release = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
            "ID_LIKE=debian",
        ]
    )
    if sys.version_info >= (3, 10):
        # mock platform.freedesktop_os_release()
        create_command.tools.platform.freedesktop_os_release = MagicMock(
            return_value=parse_freedesktop_os_release(os_release)
        )
    else:
        # For Pre Python3.10, mock the /etc/release file
        create_file(tmp_path / "os-release", os_release)
        create_command.tools.ETC_OS_RELEASE = tmp_path / "os-release"

    # Finalize the app config
    create_command.finalize_app_config(first_app_config)

    # The app's image, vendor and codename have been constructed from the target image
    assert first_app_config.target_image == "somevendor:surprising"
    assert first_app_config.target_vendor == "somevendor"
    assert first_app_config.target_codename == "surprising"
    assert first_app_config.target_vendor_base == "debian"

    # For tests of other properties merged in finalization, see
    # test_properties


def test_nodocker_non_freedesktop(create_command, first_app_config, tmp_path):
    """If the system isn't FreeDesktop compliant raise an error."""
    # Build the app without docker
    create_command.target_image = None
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    if sys.version_info >= (3, 10):
        # mock platform.freedesktop_os_release()
        create_command.tools.platform.freedesktop_os_release = MagicMock(
            side_effect=FileNotFoundError
        )
    else:
        # For Pre Python3.10, mock the /etc/release file
        # but don't create the file
        create_command.tools.ETC_OS_RELEASE = tmp_path / "os-release"

    # Finalize the app config
    with pytest.raises(
        BriefcaseCommandError,
        match=r"Could not find the /etc/os-release file. Is this a FreeDesktop-compliant Linux distribution\?",
    ):
        create_command.finalize_app_config(first_app_config)


def test_docker_arch_with_user_mapping(create_command, first_app_config, tmp_path):
    """If Docker is mapping users and the host system is Arch, an error is raised."""
    # Build the app on a specific target
    create_command.target_image = "somearch:surprising"
    create_command.tools.host_os = "Linux"
    create_command.tools.docker = MagicMock()
    create_command.tools.docker.is_user_mapped = True
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response for an Arch /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=arch",
            "VERSION_ID=20230625.0.160368",
        ]
    )

    # Finalize the app config
    with pytest.raises(
        BriefcaseCommandError,
        match="Briefcase cannot use this Docker installation",
    ):
        create_command.finalize_app_config(first_app_config)


def test_docker_arch_with_user_mapping_macOS(
    create_command, first_app_config, tmp_path
):
    """If we're on macOS, and the host system is Arch, we can finalize even though macOS
    does user mapping."""
    # Build the app on a specific target
    create_command.target_image = "somearch:surprising"
    create_command.tools.host_os = "Darwin"
    create_command.tools.docker = MagicMock()
    create_command.tools.docker.is_user_mapped = True
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response for an Arch /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=arch",
            "VERSION_ID=20230625.0.160368",
        ]
    )

    # Finalize the app config
    create_command.finalize_app_config(first_app_config)

    # The app's image, vendor and codename have been constructed from the target image
    assert first_app_config.target_image == "somearch:surprising"
    assert first_app_config.target_vendor == "arch"
    assert first_app_config.target_codename == "20230625"
    assert first_app_config.target_vendor_base == "arch"


def test_docker_arch_without_user_mapping(create_command, first_app_config, tmp_path):
    """If Docker is *not* mapping users and the host system is Arch, an error is not
    raised."""
    # Build the app on a specific target
    create_command.target_image = "somearch:surprising"
    create_command.tools.host_os = "Linux"
    create_command.tools.docker = MagicMock()
    create_command.tools.docker.is_user_mapped = False
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response for an Arch /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=arch",
            "VERSION_ID=20230625.0.160368",
        ]
    )

    # Finalize the app config
    create_command.finalize_app_config(first_app_config)

    # The app's image, vendor and codename have been constructed from the target image
    assert first_app_config.target_image == "somearch:surprising"
    assert first_app_config.target_vendor == "arch"
    assert first_app_config.target_codename == "20230625"
    assert first_app_config.target_vendor_base == "arch"


def test_properties(create_command, first_app_config):
    """The final app config is the result of merging target properties, plus other
    derived properties."""
    # Run this test as "docker"; however, the things we're testing aren't docker specific.
    create_command.target_image = "somevendor:surprising"
    create_command.tools.docker = MagicMock()
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response from checking /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
            "ID_LIKE=debian",
        ]
    )

    # Augment the app config with some extra attributes
    first_app_config.surprise_0 = "AAAA"
    first_app_config.surprise_1 = "BBBB"
    first_app_config.surprise_2 = "CCCC"
    first_app_config.surprise_3 = "DDDD"

    first_app_config.debian = {
        "surprise_1": "1111",
        "surprise_2": "1112",
        "surprise_3": "1113",
        "surprise_4": "1114",
    }
    first_app_config.somevendor = {
        "surprise_2": "2222",
        "surprise_3": "2223",
        "surprise_5": "2225",
        # A version config that will override
        "surprising": {
            "surprise_3": "3333",
            "surprise_6": "3336",
        },
        # A version config that will be ignored
        "normal": {
            "surprise_1": "XXXX",
        },
    }
    # A different vendor and version that will be ignored
    first_app_config.ubuntu = {
        "surprise_1": "YYYY",
        "jammy": {
            "surprise_1": "ZZZZ",
        },
    }

    create_command.finalize_app_config(first_app_config)

    # The target's config attributes have been merged into the app
    # Base app properties that aren't overwritten persist
    assert first_app_config.surprise_0 == "AAAA"
    # Properties can be overwritten at the base vendor level
    assert first_app_config.surprise_1 == "1111"
    # Properties can be overwritten at the vendor level
    assert first_app_config.surprise_2 == "2222"
    # Properties can be overwritten at the version level
    assert first_app_config.surprise_3 == "3333"

    # New properties can be defined at the base vendor level
    assert first_app_config.surprise_4 == "1114"
    # New properties can be defined at the vendor level
    assert first_app_config.surprise_5 == "2225"
    # New properties can be defined at the version level
    assert first_app_config.surprise_6 == "3336"

    # The glibc version was determined
    assert first_app_config.glibc_version == "2.42"

    # Since it's system python, the python version is 3
    assert first_app_config.python_version_tag == "3"


def test_properties_unknown_basevendor(create_command, first_app_config):
    """If the base vendor can't be identified, the merge still succeeds."""
    # Run this test as "docker"; however, the things we're testing aren't docker specific.
    create_command.target_image = "somevendor:surprising"
    create_command.tools.docker = MagicMock()
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response from checking /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
        ]
    )

    # Augment the app config with some extra attributes
    first_app_config.surprise_0 = "AAAA"
    first_app_config.surprise_1 = "BBBB"
    first_app_config.surprise_2 = "CCCC"
    first_app_config.surprise_3 = "DDDD"

    first_app_config.somevendor = {
        "surprise_2": "2222",
        "surprise_3": "2223",
        "surprise_5": "2225",
        # A version config that will override
        "surprising": {
            "surprise_3": "3333",
            "surprise_6": "3336",
        },
        # A version config that will be ignored
        "normal": {
            "surprise_1": "XXXX",
        },
    }
    # A different vendor and version that will be ignored
    first_app_config.ubuntu = {
        "surprise_1": "YYYY",
        "jammy": {
            "surprise_1": "ZZZZ",
        },
    }

    create_command.finalize_app_config(first_app_config)

    # The target's config attributes have been merged into the app
    # Base app properties that aren't overwritten persist
    assert first_app_config.surprise_0 == "AAAA"
    assert first_app_config.surprise_1 == "BBBB"
    # Properties can be overwritten at the vendor level
    assert first_app_config.surprise_2 == "2222"
    # Properties can be overwritten at the version level
    assert first_app_config.surprise_3 == "3333"

    # The glibc version was determined
    assert first_app_config.glibc_version == "2.42"

    # Since it's system python, the python version is 3
    assert first_app_config.python_version_tag == "3"


def test_properties_no_basevendor_config(create_command, first_app_config):
    """If there's no basevendor config, the merge still succeeds."""
    # Run this test as "docker"; however, the things we're testing aren't docker specific.
    create_command.target_image = "somevendor:surprising"
    create_command.tools.docker = MagicMock()
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response from checking /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
            "ID_LIKE=debian",
        ]
    )

    # Augment the app config with some extra attributes
    first_app_config.surprise_0 = "AAAA"
    first_app_config.surprise_1 = "BBBB"
    first_app_config.surprise_2 = "CCCC"
    first_app_config.surprise_3 = "DDDD"

    first_app_config.somevendor = {
        "surprise_2": "2222",
        "surprise_3": "2223",
        "surprise_5": "2225",
        # A version config that will override
        "surprising": {
            "surprise_3": "3333",
            "surprise_6": "3336",
        },
        # A version config that will be ignored
        "normal": {
            "surprise_1": "XXXX",
        },
    }
    # A different vendor and version that will be ignored
    first_app_config.ubuntu = {
        "surprise_1": "YYYY",
        "jammy": {
            "surprise_1": "ZZZZ",
        },
    }

    create_command.finalize_app_config(first_app_config)

    # The target's config attributes have been merged into the app
    # Base app properties that aren't overwritten persist
    assert first_app_config.surprise_0 == "AAAA"
    assert first_app_config.surprise_1 == "BBBB"
    # Properties can be overwritten at the vendor level
    assert first_app_config.surprise_2 == "2222"
    # Properties can be overwritten at the version level
    assert first_app_config.surprise_3 == "3333"

    # The glibc version was determined
    assert first_app_config.glibc_version == "2.42"

    # Since it's system python, the python version is 3
    assert first_app_config.python_version_tag == "3"


def test_properties_no_vendor(create_command, first_app_config):
    """If there's no vendor-specific config, the merge succeeds."""
    # Run this test as "docker"; however, the things we're testing aren't docker specific.
    create_command.target_image = "somevendor:surprising"
    create_command.tools.docker = MagicMock()
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response from checking /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
            "ID_LIKE=debian",
        ]
    )

    # Augment the app config with some extra attributes
    first_app_config.surprise_0 = "AAAA"
    first_app_config.surprise_1 = "BBBB"
    first_app_config.surprise_2 = "CCCC"
    first_app_config.surprise_3 = "DDDD"

    first_app_config.debian = {
        "surprise_1": "1111",
        "surprise_2": "1112",
        "surprise_3": "1113",
    }
    # A different vendor and version that will be ignored
    first_app_config.ubuntu = {
        "surprise_1": "YYYY",
        "jammy": {
            "surprise_1": "ZZZZ",
        },
    }

    create_command.finalize_app_config(first_app_config)

    # The target's config attributes have been merged into the app
    # Base app properties that aren't overwritten persist
    assert first_app_config.surprise_0 == "AAAA"
    # Properties can be overwritten at the base vendor level
    assert first_app_config.surprise_1 == "1111"
    assert first_app_config.surprise_2 == "1112"
    assert first_app_config.surprise_3 == "1113"

    # The glibc version was determined
    assert first_app_config.glibc_version == "2.42"

    # Since it's system python, the python version is 3
    assert first_app_config.python_version_tag == "3"


def test_properties_no_version(create_command, first_app_config):
    """If there's no version-specific config, the merge succeeds."""
    # Run this test as "docker"; however, the things we're testing aren't docker specific.
    create_command.target_image = "somevendor:surprising"
    create_command.tools.docker = MagicMock()
    create_command.target_glibc_version = MagicMock(return_value="2.42")

    # Mock a minimal response from checking /etc/os-release
    create_command.tools.docker.check_output.return_value = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
            "ID_LIKE=debian",
        ]
    )

    # Augment the app config with some extra attributes
    first_app_config.surprise_0 = "AAAA"
    first_app_config.surprise_1 = "BBBB"
    first_app_config.surprise_2 = "CCCC"
    first_app_config.surprise_3 = "DDDD"

    first_app_config.debian = {
        "surprise_1": "1111",
        "surprise_2": "1112",
        "surprise_3": "1113",
    }
    first_app_config.somevendor = {
        "surprise_2": "2222",
        "surprise_3": "2223",
        # A version config that will be ignored
        "normal": {
            "surprise_1": "XXXX",
        },
    }
    # A different vendor and version that will be ignored
    first_app_config.ubuntu = {
        "surprise_1": "YYYY",
        "jammy": {
            "surprise_1": "ZZZZ",
        },
    }

    create_command.finalize_app_config(first_app_config)

    # The target's config attributes have been merged into the app
    # Base app properties that aren't overwritten persist
    assert first_app_config.surprise_0 == "AAAA"
    # Properties can be overwritten at the base vendor level
    assert first_app_config.surprise_1 == "1111"
    # Properties can be overwritten at the vendor level
    assert first_app_config.surprise_2 == "2222"
    assert first_app_config.surprise_3 == "2223"

    # The glibc version was determined
    assert first_app_config.glibc_version == "2.42"

    # Since it's system python, the python version is 3
    assert first_app_config.python_version_tag == "3"


def test_passive_mixin(first_app_config, tmp_path):
    """An app using the PassiveMixin can be finalized."""
    run_command = LinuxSystemRunCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )

    # Build the app without docker
    run_command.target_image = None
    run_command.target_glibc_version = MagicMock(return_value="2.42")

    os_release = "\n".join(
        [
            "ID=somevendor",
            "VERSION_CODENAME=surprising",
            "ID_LIKE=debian",
        ]
    )
    if sys.version_info >= (3, 10):
        # mock platform.freedesktop_os_release()
        run_command.tools.platform.freedesktop_os_release = MagicMock(
            return_value=parse_freedesktop_os_release(os_release)
        )
    else:
        # For Pre Python3.10, mock the /etc/release file
        create_file(tmp_path / "os-release", os_release)
        run_command.tools.ETC_OS_RELEASE = tmp_path / "os-release"

    # Finalize the app config
    run_command.finalize_app_config(first_app_config)

    # The app's image, vendor and codename have been constructed from the target image
    assert first_app_config.target_image == "somevendor:surprising"
    assert first_app_config.target_vendor == "somevendor"
    assert first_app_config.target_codename == "surprising"
    assert first_app_config.target_vendor_base == "debian"

    # For tests of other properties merged in finalization, see
    # test_properties



import subprocess
from pathlib import Path
from unittest import mock

import pytest

from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.base import ToolCache
from briefcase.integrations.subprocess import Subprocess
from briefcase.integrations.xcode import DeviceState, get_device_state


@pytest.fixture
def mock_tools(mock_tools) -> ToolCache:
    mock_tools.subprocess = Subprocess(mock_tools)
    mock_tools.subprocess._subprocess = mock.MagicMock(spec_set=subprocess)
    mock_tools.subprocess.check_output = mock.MagicMock()
    return mock_tools


def simctl_result(name):
    """Load a simctl result file from the sample directory, and return the content."""
    filename = Path(__file__).parent / "simctl" / f"{name}.json"
    with filename.open(encoding="utf-8") as f:
        return f.read()


def test_simctl_missing(mock_tools):
    """If simctl is missing or fails to start, an exception is raised."""
    mock_tools.subprocess.check_output.side_effect = subprocess.CalledProcessError(
        cmd=["xcrun", "simctl", "list", "-j"], returncode=1
    )

    with pytest.raises(BriefcaseCommandError, match="Unable to run xcrun simctl."):
        get_device_state(mock_tools, "2D3503A3-6EB9-4B37-9B17-C7EFEF2FA32D")


def test_simctl_output_parse_error(mock_tools):
    """If parsing simctl JSON output fails, an exception is raised."""
    mock_tools.subprocess.check_output.return_value = "this is not JSON"

    with pytest.raises(
        BriefcaseCommandError, match="Unable to parse output of xcrun simctl"
    ):
        get_device_state(mock_tools, "2D3503A3-6EB9-4B37-9B17-C7EFEF2FA32D")


def test_unknown_device(mock_tools):
    """If you ask for an invalid device UDID, an exception is raised."""
    mock_tools.subprocess.check_output.return_value = simctl_result("no-devices")

    with pytest.raises(BriefcaseCommandError):
        get_device_state(mock_tools, "dead-beef-dead-beef")


def test_known_device_booted(mock_tools):
    """A valid, booted device can be inspected."""
    mock_tools.subprocess.check_output.return_value = simctl_result(
        "single-device-booted"
    )

    state = get_device_state(mock_tools, "2D3503A3-6EB9-4B37-9B17-C7EFEF2FA32D")

    assert state == DeviceState.BOOTED


def test_known_device_shutdown(mock_tools):
    """A valid, shut down device can be inspected."""
    mock_tools.subprocess.check_output.return_value = simctl_result(
        "single-device-shutdown"
    )

    state = get_device_state(mock_tools, "2D3503A3-6EB9-4B37-9B17-C7EFEF2FA32D")

    assert state == DeviceState.SHUTDOWN


def test_known_device_shutting_down(mock_tools):
    """A valid device that is shutting down can be inspected."""
    mock_tools.subprocess.check_output.return_value = simctl_result(
        "single-device-shutting-down"
    )

    state = get_device_state(mock_tools, "2D3503A3-6EB9-4B37-9B17-C7EFEF2FA32D")

    assert state == DeviceState.SHUTTING_DOWN


def test_known_device_unknown_status(mock_tools):
    """If simctl returns something unexpected as status, we can recover."""
    mock_tools.subprocess.check_output.return_value = simctl_result(
        "single-device-unknown"
    )

    state = get_device_state(mock_tools, "2D3503A3-6EB9-4B37-9B17-C7EFEF2FA32D")

    assert state == DeviceState.UNKNOWN



import subprocess
from datetime import datetime
from unittest.mock import Mock

import pytest

from briefcase.exceptions import BriefcaseCommandError


@pytest.mark.parametrize(
    "device_output, expected_datetime",
    [
        ("2023-07-12 09:28:04", datetime(2023, 7, 12, 9, 28, 4)),
        ("2023-07-12 09:28:04\n", datetime(2023, 7, 12, 9, 28, 4)),
        ("2023-7-12 9:28:04", datetime(2023, 7, 12, 9, 28, 4)),
        ("2023-12-2 14:28:04", datetime(2023, 12, 2, 14, 28, 4)),
    ],
)
def test_datetime_success(adb, device_output, expected_datetime):
    """adb.datetime() returns `datetime` for device."""
    adb.run = Mock(return_value=device_output)

    assert adb.datetime() == expected_datetime
    adb.run.assert_called_once_with("shell", "date", "+'%Y-%m-%d %H:%M:%S'")


def test_datetime_failure_call(adb):
    """adb.datetime() fails in subprocess call."""
    adb.run = Mock(
        side_effect=subprocess.CalledProcessError(returncode=1, cmd="adb shell ...")
    )

    with pytest.raises(
        BriefcaseCommandError,
        match="Error obtaining device date/time.",
    ):
        adb.datetime()


def test_datetime_failure_bad_value(adb):
    """adb.datetime() fails in output conversion."""
    adb.run = Mock(return_value="the date is jan 1 1970")

    with pytest.raises(
        BriefcaseCommandError,
        match="Error obtaining device date/time.",
    ):
        adb.datetime()



from unittest import mock

import pytest

from briefcase.commands.base import BaseCommand
from briefcase.console import Console, Log
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.macOS.app import macOSAppMixin, macOSCreateMixin
from tests.utils import DummyConsole


class DummyInstallCommand(macOSAppMixin, macOSCreateMixin, BaseCommand):
    """A dummy command to expose package installation capabilities."""

    command = "install"

    def __init__(self, base_path, **kwargs):
        kwargs.setdefault("logger", Log())
        kwargs.setdefault("console", Console())
        super().__init__(base_path=base_path / "base_path", **kwargs)
        self.tools.input = DummyConsole()


@pytest.fixture
def dummy_command(tmp_path):
    cmd = DummyInstallCommand(base_path=tmp_path)

    cmd.tools.subprocess = mock.MagicMock(spec_set=Subprocess)

    return cmd



from unittest.mock import MagicMock

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseConfigError, UnsupportedHostError
from briefcase.platforms.linux.appimage import LinuxAppImageCreateCommand


@pytest.fixture
def create_command(first_app_config, tmp_path):
    command = LinuxAppImageCreateCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.host_arch = "x86_64"
    return command


def test_default_options(create_command):
    """The default options are as expected."""
    options, overrides = create_command.parse_options([])

    assert options == {}
    assert overrides == {}

    assert create_command.use_docker


def test_options(create_command):
    """The extra options can be parsed."""
    options, overrides = create_command.parse_options(["--no-docker"])

    assert options == {}
    assert overrides == {}

    assert not create_command.use_docker


@pytest.mark.parametrize("host_os", ["Windows", "WeirdOS"])
def test_unsupported_host_os_with_docker(create_command, host_os):
    """Error raised for an unsupported OS when using Docker."""
    create_command.use_docker = True
    create_command.tools.host_os = host_os

    with pytest.raises(
        UnsupportedHostError,
        match="Linux AppImages can only be built on Linux, or on macOS using Docker.",
    ):
        create_command()


@pytest.mark.parametrize("host_os", ["Darwin", "Windows", "WeirdOS"])
def test_unsupported_host_os_without_docker(create_command, host_os):
    """Error raised for an unsupported OS when not using Docker."""
    create_command.use_docker = False
    create_command.tools.host_os = host_os

    with pytest.raises(
        UnsupportedHostError,
        match="Linux AppImages can only be built on Linux, or on macOS using Docker.",
    ):
        create_command()


def test_finalize_docker(create_command, first_app_config, capsys):
    """No warning is generated when building an AppImage in Docker."""
    create_command.use_docker = True

    create_command.finalize_app_config(first_app_config)

    stdout = capsys.readouterr().out
    # Warning message was not recorded
    assert "WARNING: Building a Local AppImage!" not in stdout

    # Generic appimage warning *was* recorded
    assert "WARNING: Use of AppImage is not recommended!" in stdout


def test_finalize_nodocker(create_command, first_app_config, capsys):
    """A warning is generated when building an AppImage outside Docker."""
    create_command.use_docker = False

    create_command.finalize_app_config(first_app_config)

    stdout = capsys.readouterr().out
    # Warning message was recorded
    assert "WARNING: Building a Local AppImage!" in stdout

    # Generic appimage warning *was* recorded
    assert "WARNING: Use of AppImage is not recommended!" in stdout


@pytest.mark.parametrize(
    "manylinux, tag, host_os, host_arch, is_user_mapped, context",
    [
        # Fallback.
        (None, None, "Linux", "x86_64", False, {"use_non_root_user": True}),
        # Linux on x86_64 architecture, all versions
        # Explicit tag
        (
            "manylinux1",
            "2023-03-05-271004f",
            "Linux",
            "x86_64",
            True,
            {
                "manylinux_image": "manylinux1_x86_64:2023-03-05-271004f",
                "vendor_base": "centos",
                "use_non_root_user": False,
            },
        ),
        # Explicit latest
        (
            "manylinux2010",
            "latest",
            "Linux",
            "x86_64",
            False,
            {
                "manylinux_image": "manylinux2010_x86_64:latest",
                "vendor_base": "centos",
                "use_non_root_user": True,
            },
        ),
        # Implicit latest
        (
            "manylinux2014",
            None,
            "Linux",
            "x86_64",
            True,
            {
                "manylinux_image": "manylinux2014_x86_64:latest",
                "vendor_base": "centos",
                "use_non_root_user": False,
            },
        ),
        (
            "manylinux_2_24",
            None,
            "Linux",
            "x86_64",
            True,
            {
                "manylinux_image": "manylinux_2_24_x86_64:latest",
                "vendor_base": "debian",
                "use_non_root_user": False,
            },
        ),
        (
            "manylinux_2_28",
            None,
            "Linux",
            "x86_64",
            False,
            {
                "manylinux_image": "manylinux_2_28_x86_64:latest",
                "vendor_base": "almalinux",
                "use_non_root_user": True,
            },
        ),
        # Linux on i686 hardware
        (
            "manylinux_2_28",
            None,
            "Linux",
            "i686",
            False,
            {
                "manylinux_image": "manylinux_2_28_i686:latest",
                "vendor_base": "almalinux",
                "use_non_root_user": True,
            },
        ),
        # Linux on aarch64 hardware
        (
            "manylinux_2_28",
            None,
            "Linux",
            "aarch64",
            False,
            {
                "manylinux_image": "manylinux_2_28_aarch64:latest",
                "vendor_base": "almalinux",
                "use_non_root_user": True,
            },
        ),
        # Linux on arm hardware
        (
            "manylinux_2_28",
            None,
            "Linux",
            "armv7l",
            False,
            {
                "manylinux_image": "manylinux_2_28_armhf:latest",
                "vendor_base": "almalinux",
                "use_non_root_user": True,
            },
        ),
        # macOS on x86_64
        (
            "manylinux2014",
            None,
            "Darwin",
            "x86_64",
            True,
            {
                "manylinux_image": "manylinux2014_x86_64:latest",
                "vendor_base": "centos",
                "use_non_root_user": False,
            },
        ),
    ],
)
def test_output_format_template_context(
    create_command,
    first_app_config,
    manylinux,
    tag,
    host_os,
    host_arch,
    is_user_mapped,
    context,
):
    """The template context reflects the manylinux name, tag and architecture."""
    # Mock Docker user mapping setting for `use_non_root_user`
    create_command.tools.docker = MagicMock()
    create_command.tools.docker.is_user_mapped = is_user_mapped

    if manylinux:
        first_app_config.manylinux = manylinux
    if tag:
        first_app_config.manylinux_image_tag = tag

    create_command.tools.host_os = host_os
    create_command.tools.host_arch = host_arch

    assert create_command.output_format_template_context(first_app_config) == context


def test_output_format_template_context_bad_tag(create_command, first_app_config):
    """An unknown manylinux tag raises an error."""
    first_app_config.manylinux = "unknown"
    with pytest.raises(BriefcaseConfigError, match=r"Unknown manylinux tag 'unknown'"):
        assert create_command.output_format_template_context(first_app_config)


def test_output_format_no_docker(create_command, first_app_config):
    """If not using Docker, `use_non_root_user` default in template is used."""
    context = create_command.output_format_template_context(first_app_config)

    assert "use_non_root_user" not in context



import pytest

from briefcase.config import AppConfig

from .conftest import DummyCommand


@pytest.fixture
def first_app():
    return AppConfig(
        app_name="first",
        bundle="com.example",
        version="0.0.1",
        description="The first simple app",
        sources=["src/first"],
        license={"file": "LICENSE"},
    )


@pytest.fixture
def second_app():
    return AppConfig(
        app_name="second",
        bundle="com.example",
        version="0.0.2",
        description="The second simple app",
        sources=["src/second"],
        license={"file": "LICENSE"},
    )


@pytest.fixture
def base_command(tmp_path, first_app, second_app):
    return DummyCommand(
        base_path=tmp_path,
        apps={
            "first": first_app,
            "second": second_app,
        },
    )


def test_finalize_all(base_command, first_app, second_app):
    "A call to finalize verifies host, tools, and finalized all app configs"
    base_command.finalize()

    # The right sequence of things will be done
    assert base_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App config has been finalized
        ("finalize-app-config", "second"),
    ]

    # Apps are no longer in draft mode
    assert not hasattr(first_app, "__draft__")
    assert not hasattr(second_app, "__draft__")


def test_finalize_single(base_command, first_app, second_app):
    "A call to finalize verifies host, tools, and finalized all app configs"
    base_command.finalize(first_app)

    # The right sequence of things will be done
    assert base_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
    ]

    # First app is no longer in draft mode; second is
    assert not hasattr(first_app, "__draft__")
    assert hasattr(second_app, "__draft__")


def test_finalize_all_repeat(base_command, first_app, second_app):
    "Multiple calls to finalize verifies host & tools multiple times, but only once on config"
    # Finalize apps twice. This is an approximation of what happens
    # when a command chain is executed; create, update, build and run will
    # all finalize; create will finalize the app configs, each command will
    # have it's own tools verified.
    base_command.finalize()
    base_command.finalize()

    # The right sequence of things will be done
    assert base_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # App config has been finalized
        ("finalize-app-config", "second"),
        # Host OS is verified again
        ("verify-host",),
        # Tools are verified again
        ("verify-tools",),
    ]

    # Apps are no longer in draft mode
    assert not hasattr(first_app, "__draft__")
    assert not hasattr(second_app, "__draft__")


def test_finalize_single_repeat(base_command, first_app, second_app):
    "Multiple calls to finalize verifies host & tools multiple times, but finalizes app config once"

    # Finalize app twice. This is an approximation of what happens
    # when a command chain is executed; create, update, build and run will
    # all finalize; create will finalize the app config, each command will
    # have it's own tools verified.
    base_command.finalize(first_app)
    base_command.finalize(first_app)

    # The right sequence of things will be done
    assert base_command.actions == [
        # Host OS is verified
        ("verify-host",),
        # Tools are verified
        ("verify-tools",),
        # App config has been finalized
        ("finalize-app-config", "first"),
        # Host OS is verified again
        ("verify-host",),
        # Tools are verified again
        ("verify-tools",),
    ]

    # First app is no longer in draft mode; second is
    assert not hasattr(first_app, "__draft__")
    assert hasattr(second_app, "__draft__")



from __future__ import annotations

import subprocess

from briefcase.commands import (
    BuildCommand,
    CreateCommand,
    OpenCommand,
    PackageCommand,
    PublishCommand,
    RunCommand,
    UpdateCommand,
)
from briefcase.config import BaseConfig
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.xcode import Xcode
from briefcase.platforms.macOS import (
    macOSCreateMixin,
    macOSMixin,
    macOSPackageMixin,
    macOSRunMixin,
)
from briefcase.platforms.macOS.filters import XcodeBuildFilter


class macOSXcodeMixin(macOSMixin):
    output_format = "Xcode"
    supported_host_os_reason = (
        "macOS applications require the Xcode command line "
        "tools, which are only available on macOS."
    )

    def verify_tools(self):
        Xcode.verify(self.tools, min_version=(13, 0, 0))

        # Verify superclass tools *after* xcode. This ensures we get the
        # git check *after* the xcode check.
        super().verify_tools()

    def project_path(self, app):
        return self.bundle_path(app) / f"{app.formal_name}.xcodeproj"

    def binary_path(self, app):
        return self.bundle_path(app) / "build/Release" / f"{app.formal_name}.app"


class macOSXcodeCreateCommand(macOSXcodeMixin, macOSCreateMixin, CreateCommand):
    description = "Create and populate a macOS Xcode project."


class macOSXcodeOpenCommand(macOSXcodeMixin, OpenCommand):
    description = "Open an existing macOS Xcode project."


class macOSXcodeUpdateCommand(macOSXcodeCreateCommand, UpdateCommand):
    description = "Update an existing macOS Xcode project."


class macOSXcodeBuildCommand(macOSXcodeMixin, BuildCommand):
    description = "Build a macOS Xcode project."

    def build_app(self, app: BaseConfig, **kwargs):
        """Build the Xcode project for the application.

        :param app: The application to build
        """
        self.logger.info("Building Xcode project...", prefix=app.app_name)
        with self.input.wait_bar("Building..."):
            try:
                self.tools.subprocess.run(
                    [
                        "xcodebuild",
                        "-project",
                        self.project_path(app),
                        "-verbose" if self.tools.logger.is_deep_debug else "-quiet",
                        "-configuration",
                        "Release",
                        "build",
                    ],
                    check=True,
                    filter_func=(
                        None if self.tools.logger.is_deep_debug else XcodeBuildFilter()
                    ),
                )
                self.logger.info("Build succeeded.")
            except subprocess.CalledProcessError as e:
                raise BriefcaseCommandError(
                    f"Unable to build app {app.app_name}."
                ) from e


class macOSXcodeRunCommand(macOSRunMixin, macOSXcodeMixin, RunCommand):
    description = "Run a macOS app."


class macOSXcodePackageCommand(macOSPackageMixin, macOSXcodeMixin, PackageCommand):
    description = "Package a macOS app for distribution."


class macOSXcodePublishCommand(macOSXcodeMixin, PublishCommand):
    description = "Publish a macOS app."


# Declare the briefcase command bindings
create = macOSXcodeCreateCommand
update = macOSXcodeUpdateCommand
open = macOSXcodeOpenCommand
build = macOSXcodeBuildCommand
run = macOSXcodeRunCommand
package = macOSXcodePackageCommand
publish = macOSXcodePublishCommand



import pytest

from briefcase.config import make_class_name


@pytest.mark.parametrize(
    "formal_name, candidate",
    [
        # Some simple cases
        ("Hello World", "HelloWorld"),
        ("Hello World!", "HelloWorld"),
        ("Hello! World", "HelloWorld"),
        ("Hello_World", "Hello_World"),
        ("Hello-World", "HelloWorld"),
        # Startint with a number
        ("24 Jump Street", "_24JumpStreet"),  # Unicode category Nd
        # Starting with an underscore
        ("Hello_World", "Hello_World"),
        ("_Hello_World", "_Hello_World"),
        # Unicode names
        ("你好 世界", "你好世界"),
        ("Hallo Vögel", "HalloVögel"),
        ("Bonjour Garçon", "BonjourGarçon"),
        # Unicode codepoints that can be at the start of an identifier
        ("\u02EC World", "\u02ECWorld"),  # Unicode category Lm
        ("\u3006 World", "\u3006World"),  # Unicode category Lo
        ("\u3021 World", "\u3021World"),  # Unicode category Nl
        # ('\u2118 World', '\u2118World'),  # in Other_ID_Start
        # Unicode codepoints that cannot be at the start of an identifier
        ("\u20E1 World", "_\u20E1World"),  # Unicode Category Mn
        ("\u0903 World", "_\u0903World"),  # Unicode Category Mc
        ("\u2040 World", "_\u2040World"),  # Category Pc
        # ('\u00B7 World', '_\u00B7World'),  # in Other_ID_Continue
        # Characters that are converted by NFKC normalization
        ("\u2135 World", "\u05d0World"),  # Unicode category Lo
    ],
)
def test_make_class_name(formal_name, candidate):
    """A formal name can be converted into a valid class name."""
    class_name = make_class_name(formal_name)
    assert class_name == candidate



import pytest

from briefcase.commands import NewCommand
from briefcase.commands.base import full_options
from briefcase.console import Console, Log
from tests.utils import DummyConsole


class DummyNewCommand(NewCommand):
    """A dummy new command that doesn't actually do anything.

    It only serves to track which actions would be performed.
    """

    description = "Dummy new command"

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("logger", Log())
        kwargs.setdefault("console", Console())
        super().__init__(*args, apps={}, **kwargs)

        self.actions = []
        self.tools.input = DummyConsole()

    def verify_host(self):
        super().verify_host()
        self.actions.append(("verify-host",))

    def verify_tools(self):
        super().verify_tools()
        self.actions.append(("verify-tools",))

    def finalize_app_config(self, app):
        super().finalize_app_config(app=app)
        self.actions.append(("finalize-app-config", app))

    def verify_app_tools(self, app):
        super().verify_app_tools(app=app)
        self.actions.append(("verify-app-tools", app.app_name))

    def new_app(self, **kwargs):
        self.actions.append(("new", kwargs))
        return full_options({"new_state": "done"}, kwargs)


@pytest.fixture
def new_command(tmp_path):
    return DummyNewCommand(base_path=tmp_path)



import pytest


@pytest.mark.parametrize(
    "env, subprocess_env",
    [
        (None, {"DOCKER_CLI_HINTS": "false"}),
        ({}, {"DOCKER_CLI_HINTS": "false"}),
        ({"ENV_VAR": "VALUE"}, {"ENV_VAR": "VALUE", "DOCKER_CLI_HINTS": "false"}),
        (
            {"ENV_VAR": "VALUE", "DOCKER_CLI_HINTS": "true"},
            {
                "ENV_VAR": "VALUE",
                "DOCKER_CLI_HINTS": "true",
            },
        ),
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_subprocess_env(mock_tools, env, subprocess_env):
    """The env for subprocess calls is set properly."""
    assert mock_tools.docker.subprocess_env(env) == subprocess_env



import pytest

from briefcase.exceptions import UnsupportedHostError
from briefcase.integrations.subprocess import NativeAppContext, Subprocess


def test_short_circuit(mock_tools, first_app_config):
    """Tool is not created if already cached."""
    mock_tools[first_app_config].app_context = "tool"

    tool = NativeAppContext.verify(mock_tools, first_app_config)

    assert tool == "tool"
    assert tool == mock_tools[first_app_config].app_context


def test_unsupported_os(mock_tools):
    """When host OS is not supported, an error is raised."""
    mock_tools.host_os = "wonky"

    with pytest.raises(
        UnsupportedHostError,
        match=f"{NativeAppContext.name} is not supported on wonky",
    ):
        NativeAppContext.verify(mock_tools, app=object())


def test_verify(mock_tools, first_app_config):
    """Subprocess verify always returns/sets subprocess tool."""
    native_app_context_tool = NativeAppContext.verify(mock_tools, first_app_config)

    assert isinstance(native_app_context_tool, Subprocess)
    assert native_app_context_tool is mock_tools[first_app_config].app_context
    # native app context should be the default subprocess tool
    assert native_app_context_tool is mock_tools.subprocess



import subprocess
from unittest.mock import call

import pytest


@pytest.mark.usefixtures("mock_docker")
def test_check_output(mock_tools, sub_check_output_kw):
    """A command can be invoked on a bare Docker image."""
    # mock image already being cached in Docker
    mock_tools.subprocess._subprocess.check_output.side_effect = [
        "1ed313b0551f",
        "output",
    ]

    # Run the command in a container
    mock_tools.docker.check_output(["cmd", "arg1", "arg2"], image_tag="ubuntu:jammy")

    mock_tools.subprocess._subprocess.check_output.assert_has_calls(
        [
            # Verify image is cached in Docker
            call(
                ["docker", "images", "-q", "ubuntu:jammy"],
                env={"PROCESS_ENV_VAR": "VALUE", "DOCKER_CLI_HINTS": "false"},
                **sub_check_output_kw,
            ),
            # Run command in Docker using image
            call(
                ["docker", "run", "--rm", "ubuntu:jammy", "cmd", "arg1", "arg2"],
                env={"PROCESS_ENV_VAR": "VALUE", "DOCKER_CLI_HINTS": "false"},
                **sub_check_output_kw,
            ),
        ]
    )


@pytest.mark.usefixtures("mock_docker")
def test_check_output_fail(mock_tools, sub_check_output_kw):
    """Any subprocess errors are passed back through directly."""
    # mock image already being cached in Docker and check_output() call fails
    mock_tools.subprocess._subprocess.check_output.side_effect = [
        "1ed313b0551f",
        subprocess.CalledProcessError(returncode=1, cmd=["cmd", "arg1", "arg2"]),
    ]

    # The CalledProcessError surfaces from Docker().check_output()
    with pytest.raises(subprocess.CalledProcessError):
        mock_tools.docker.check_output(
            ["cmd", "arg1", "arg2"], image_tag="ubuntu:jammy"
        )

    mock_tools.subprocess._subprocess.check_output.assert_has_calls(
        [
            # Verify image is cached in Docker
            call(
                ["docker", "images", "-q", "ubuntu:jammy"],
                env={"PROCESS_ENV_VAR": "VALUE", "DOCKER_CLI_HINTS": "false"},
                **sub_check_output_kw,
            ),
            # Command errors in Docker using image
            call(
                ["docker", "run", "--rm", "ubuntu:jammy", "cmd", "arg1", "arg2"],
                env={"PROCESS_ENV_VAR": "VALUE", "DOCKER_CLI_HINTS": "false"},
                **sub_check_output_kw,
            ),
        ]
    )
